@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: transmissioncli.c 1770 2007-04-21 12:37:12Z livings124 $
+ * $Id: transmissioncli.c 3486 2007-10-20 22:07:21Z charles $
  *
  * Copyright (c) 2005-2006 Transmission authors and contributors
  *
@@ -28,28 +28,55 @@
 #include <unistd.h>
 #include <getopt.h>
 #include <signal.h>
-#include <transmission.h>
-#ifdef SYS_BEOS
-#include <kernel/OS.h>
-#define usleep snooze
+
+#include <libtransmission/transmission.h>
+#include <libtransmission/makemeta.h>
+
+#ifdef __BEOS__
+    #include <kernel/OS.h>
+    #define wait_msecs(N)  snooze( (N) * 1000 )
+    #define wait_secs(N)   sleep( (N) )
+#elif defined(WIN32)
+    #include <windows.h>
+    #define wait_msecs(N)  Sleep( (N) )
+    #define wait_secs(N)   Sleep( (N) * 1000 )
+#else
+    #define wait_msecs(N)  usleep( (N) * 1000 )
+    #define wait_secs(N)   sleep( (N) )
 #endif
 
-#define USAGE \
-"Usage: %s [options] file.torrent [options]\n\n" \
-"Options:\n" \
-"  -d, --download <int> Maximum download rate (-1 = no limit, default = -1)\n"\
-"  -f, --finish <shell script> Command you wish to run on completion\n" \
-"  -h, --help           Print this help and exit\n" \
-"  -i, --info           Print metainfo and exit\n" \
-"  -n  --nat-traversal  Attempt NAT traversal using NAT-PMP or UPnP IGD\n" \
-"  -p, --port <int>     Port we should listen on (default = %d)\n" \
-"  -s, --scrape         Print counts of seeders/leechers and exit\n" \
-"  -u, --upload <int>   Maximum upload rate (-1 = no limit, default = 20)\n" \
-"  -v, --verbose <int>  Verbose level (0 to 2, default = 0)\n"
+/* macro to shut up "unused parameter" warnings */
+#ifdef __GNUC__
+#define UNUSED                  __attribute__((unused))
+#else
+#define UNUSED
+#endif
+
+const char * USAGE =
+"Usage: %s [options] file.torrent [options]\n\n"
+"Options:\n"
+"  -c, --create-from <file>  Create torrent from the specified source file.\n"
+"  -a, --announce <url> Used in conjunction with -c.\n"
+"  -r, --private        Used in conjunction with -c.\n"
+"  -m, --comment <text> Adds an optional comment when creating a torrent.\n"
+"  -d, --download <int> Maximum download rate (-1 = no limit, default = -1)\n"
+"  -f, --finish <shell script> Command you wish to run on completion\n" 
+"  -h, --help           Print this help and exit\n" 
+"  -i, --info           Print metainfo and exit\n"
+"  -n  --nat-traversal  Attempt NAT traversal using NAT-PMP or UPnP IGD\n"
+"  -p, --port <int>     Port we should listen on (default = %d)\n"
+#if 0
+"  -s, --scrape         Print counts of seeders/leechers and exit\n"
+#endif
+"  -u, --upload <int>   Maximum upload rate (-1 = no limit, default = 20)\n"
+"  -v, --verbose <int>  Verbose level (0 to 2, default = 0)\n";
 
 static int           showHelp      = 0;
 static int           showInfo      = 0;
+#if 0
 static int           showScrape    = 0;
+#endif
+static int           isPrivate     = 0;
 static int           verboseLevel  = 0;
 static int           bindPort      = TR_DEFAULT_PORT;
 static int           uploadLimit   = 20;
@@ -57,9 +84,13 @@ static int           downloadLimit = -1;
 static char          * torrentPath = NULL;
 static int           natTraversal  = 0;
 static sig_atomic_t  gotsig        = 0;
-static tr_torrent_t  * tor;
+static sig_atomic_t  manualUpdate  = 0;
+static tr_torrent    * tor;
 
 static char          * finishCall   = NULL;
+static char          * announce     = NULL;
+static char          * sourceFile   = NULL;
+static char          * comment      = NULL;
 
 static int  parseCommandLine ( int argc, char ** argv );
 static void sigHandler       ( int signal );
@@ -76,15 +107,23 @@ char * getStringRatio( float ratio )
 
 #define LINEWIDTH 80
 
+static void
+torrentStateChanged( tr_torrent   * torrent UNUSED,
+                     cp_status_t    status UNUSED,
+                     void         * user_data UNUSED )
+{
+    system( finishCall );
+}
+
 int main( int argc, char ** argv )
 {
     int i, error;
-    tr_handle_t  * h;
-    tr_stat_t    * s;
-    tr_handle_status_t * hstat;
+    tr_handle  * h;
+    const tr_stat    * s;
+    tr_handle_status * hstat;
 
-    printf( "Transmission %s (%d) - http://transmission.m0k.org/\n\n",
-            VERSION_STRING, VERSION_REVISION );
+    printf( "Transmission %s - http://transmission.m0k.org/\n\n",
+            LONG_VERSION_STRING );
 
     /* Get options */
     if( parseCommandLine( argc, argv ) )
@@ -123,8 +162,22 @@ int main( int argc, char ** argv )
     /* Initialize libtransmission */
     h = tr_init( "cli" );
 
+    if( sourceFile && *sourceFile ) /* creating a torrent */
+    {
+        int ret;
+        tr_metainfo_builder * builder = tr_metaInfoBuilderCreate( h, sourceFile );
+        tr_makeMetaInfo( builder, torrentPath, announce, comment, isPrivate );
+        while( !builder->isDone ) {
+            wait_msecs( 1 );
+            printf( "." );
+        }
+        ret = !builder->failed;
+        tr_metaInfoBuilderFree( builder );
+        return ret;
+    }
+
     /* Open and parse torrent file */
-    if( !( tor = tr_torrentInit( h, torrentPath, NULL, 0, &error ) ) )
+    if( !( tor = tr_torrentInit( h, torrentPath, ".", 0, &error ) ) )
     {
         printf( "Failed opening torrent file `%s'\n", torrentPath );
         tr_close( h );
@@ -133,7 +186,7 @@ int main( int argc, char ** argv )
 
     if( showInfo )
     {
-        tr_info_t * info = tr_torrentInfo( tor );
+        const tr_info * info = tr_torrentInfo( tor );
 
         s = tr_torrentStat( tor );
 
@@ -158,7 +211,7 @@ int main( int argc, char ** argv )
         {
             printf( "creator:  %s\n", info->creator );
         }
-        if( TR_FLAG_PRIVATE & info->flags )
+        if( info->isPrivate )
         {
             printf( "private flag set\n" );
         }
@@ -172,6 +225,7 @@ int main( int argc, char ** argv )
         goto cleanup;
     }
 
+#if 0
     if( showScrape )
     {
         int seeders, leechers, downloaded;
@@ -188,63 +242,80 @@ int main( int argc, char ** argv )
 
         goto cleanup;
     }
+#endif
 
     signal( SIGINT, sigHandler );
+    signal( SIGHUP, sigHandler );
 
     tr_setBindPort( h, bindPort );
-    tr_setGlobalUploadLimit( h, uploadLimit );
-    tr_setGlobalDownloadLimit( h, downloadLimit );
+  
+    tr_setGlobalSpeedLimit   ( h, TR_UP,   uploadLimit );
+    tr_setUseGlobalSpeedLimit( h, TR_UP,   uploadLimit > 0 );
+    tr_setGlobalSpeedLimit   ( h, TR_DOWN, downloadLimit );
+    tr_setUseGlobalSpeedLimit( h, TR_DOWN, downloadLimit > 0 );
 
     tr_natTraversalEnable( h, natTraversal );
     
-    tr_torrentSetFolder( tor, "." );
+    tr_torrentSetStatusCallback( tor, torrentStateChanged, NULL );
     tr_torrentStart( tor );
 
     for( ;; )
     {
         char string[LINEWIDTH];
         int  chars = 0;
-        int result;
 
-        sleep( 1 );
+        wait_secs( 1 );
 
         if( gotsig )
         {
             gotsig = 0;
             tr_torrentStop( tor );
+            tr_natTraversalEnable( h, 0 );
+        }
+        
+        if( manualUpdate )
+        {
+            manualUpdate = 0;
+            if ( !tr_torrentCanManualUpdate( tor ) )
+                fprintf( stderr, "\rReceived SIGHUP, but can't send a manual update now\n" );
+            else {
+                fprintf( stderr, "\rReceived SIGHUP: manual update scheduled\n" );
+                tr_manualUpdate( tor );
+            }
         }
 
         s = tr_torrentStat( tor );
 
-        if( s->status & TR_STATUS_PAUSE )
+        if( s->status & TR_STATUS_CHECK_WAIT )
         {
-            break;
+            chars = snprintf( string, sizeof string,
+                "Waiting to verify local files... %.2f %%", 100.0 * s->percentDone );
         }
         else if( s->status & TR_STATUS_CHECK )
         {
             chars = snprintf( string, sizeof string,
-                "Checking files... %.2f %%", 100.0 * s->progress );
+                "Verifying local files... %.2f %%", 100.0 * s->percentDone );
         }
         else if( s->status & TR_STATUS_DOWNLOAD )
         {
             chars = snprintf( string, sizeof string,
                 "Progress: %.2f %%, %d peer%s, dl from %d (%.2f KB/s), "
-                "ul to %d (%.2f KB/s) [%s]", 100.0 * s->progress,
-                s->peersTotal, ( s->peersTotal == 1 ) ? "" : "s",
-                s->peersUploading, s->rateDownload,
-                s->peersDownloading, s->rateUpload,
+                "ul to %d (%.2f KB/s) [%s]", 100.0 * s->percentDone,
+                s->peersConnected, ( s->peersConnected == 1 ) ? "" : "s",
+                s->peersSendingToUs, s->rateDownload,
+                s->peersGettingFromUs, s->rateUpload,
                 getStringRatio(s->ratio) );
         }
         else if( s->status & TR_STATUS_SEED )
         {
             chars = snprintf( string, sizeof string,
                 "Seeding, uploading to %d of %d peer(s), %.2f KB/s [%s]",
-                s->peersDownloading, s->peersTotal,
+                s->peersGettingFromUs, s->peersConnected,
                 s->rateUpload, getStringRatio(s->ratio) );
         }
-        else if( s->status & TR_STATUS_STOPPING )
+        else if( s->status & TR_STATUS_STOPPED )
         {
-            chars = snprintf( string, sizeof string, "Stopping..." );
+            break;
         }
         if( ( signed )sizeof string > chars )
         {
@@ -261,11 +332,6 @@ int main( int argc, char ** argv )
         {
             fprintf( stderr, "\n" );
         }
-        
-        if( tr_getFinished( tor ) )
-        {
-            result = system(finishCall);
-        }
     }
     fprintf( stderr, "\n" );
 
@@ -279,11 +345,11 @@ int main( int argc, char ** argv )
             /* Port mappings were deleted */
             break;
         }
-        usleep( 500000 );
+        wait_msecs( 500 );
     }
     
 cleanup:
-    tr_torrentClose( h, tor );
+    tr_torrentClose( tor );
     tr_close( h );
 
     return EXIT_SUCCESS;
@@ -297,16 +363,21 @@ static int parseCommandLine( int argc, char ** argv )
           { { "help",     no_argument,       NULL, 'h' },
             { "info",     no_argument,       NULL, 'i' },
             { "scrape",   no_argument,       NULL, 's' },
+            { "private",  no_argument,       NULL, 'r' },
             { "verbose",  required_argument, NULL, 'v' },
             { "port",     required_argument, NULL, 'p' },
             { "upload",   required_argument, NULL, 'u' },
             { "download", required_argument, NULL, 'd' },
             { "finish",   required_argument, NULL, 'f' },
+            { "create",   required_argument, NULL, 'c' },
+            { "comment",  required_argument, NULL, 'm' },
+            { "announce", required_argument, NULL, 'a' },
             { "nat-traversal", no_argument,  NULL, 'n' },
             { 0, 0, 0, 0} };
 
         int c, optind = 0;
-        c = getopt_long( argc, argv, "hisv:p:u:d:f:n", long_options, &optind );
+        c = getopt_long( argc, argv, "hisrv:p:u:d:f:c:m:a:n",
+                         long_options, &optind );
         if( c < 0 )
         {
             break;
@@ -319,9 +390,14 @@ static int parseCommandLine( int argc, char ** argv )
             case 'i':
                 showInfo = 1;
                 break;
+#if 0
             case 's':
                 showScrape = 1;
                 break;
+#endif
+            case 'r':
+                isPrivate = 1;
+                break;
             case 'v':
                 verboseLevel = atoi( optarg );
                 break;
@@ -337,6 +413,15 @@ static int parseCommandLine( int argc, char ** argv )
             case 'f':
                 finishCall = optarg;
                 break;
+            case 'c':
+                sourceFile = optarg;
+                break;
+            case 'm':
+                comment = optarg;
+                break;
+            case 'a':
+                announce = optarg;
+                break;
             case 'n':
                 natTraversal = 1;
                 break;
@@ -362,6 +447,10 @@ static void sigHandler( int signal )
         case SIGINT:
             gotsig = 1;
             break;
+            
+        case SIGHUP:
+            manualUpdate = 1;
+            break;
 
         default:
             break;
