@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: client.c 1751 2007-04-18 16:41:37Z joshe $
+ * $Id: client.c 3175 2007-09-25 23:59:50Z charles $
  *
  * Copyright (c) 2007 Joshua Elsasser
  *
@@ -35,14 +35,15 @@
 #include <string.h>
 #include <unistd.h>
 
+#include <libtransmission/bsdtree.h>
+#include <libtransmission/bencode.h>
+#include <libtransmission/ipcparse.h>
+#include <libtransmission/trcompat.h>
+
 #include "bsdqueue.h"
-#include "bsdtree.h"
 #include "client.h"
-#include "bencode.h"
 #include "errors.h"
-#include "ipc.h"
 #include "misc.h"
-#include "trcompat.h"
 
 /* time out server after this many seconds */
 #define SERVER_TIMEOUT          ( 15 )
@@ -51,7 +52,7 @@ struct con
 {
     int                  infd;
     int                  outfd;
-    struct ipc_info      ipc;
+    struct ipc_info    * ipc;
     struct bufferevent * evin;
     struct bufferevent * evout;
 };
@@ -134,7 +135,7 @@ client_init( struct event_base * base )
 int
 client_new_sock( const char * path )
 {
-    struct sockaddr_un sun;
+    struct sockaddr_un sa;
     int                fd;
     struct con       * con;
 
@@ -144,9 +145,9 @@ client_new_sock( const char * path )
 
     gl_proxy = 0;
 
-    bzero( &sun, sizeof sun );
-    sun.sun_family = AF_LOCAL;
-    strlcpy( sun.sun_path, path, sizeof sun.sun_path );
+    memset( &sa, 0, sizeof sa );
+    sa.sun_family = AF_LOCAL;
+    strlcpy( sa.sun_path, path, sizeof sa.sun_path );
 
     fd = socket( AF_UNIX, SOCK_STREAM, 0 );
     if( 0 > fd )
@@ -155,7 +156,7 @@ client_new_sock( const char * path )
         return -1;
     }
 
-    if( 0 > connect( fd, ( struct sockaddr * )&sun, SUN_LEN( &sun ) ) )
+    if( 0 > connect( fd, ( struct sockaddr * )&sa, SUN_LEN( &sa ) ) )
     {
         errnomsg( "failed to connect to socket file: %s", path );
         close( fd );
@@ -168,13 +169,20 @@ client_new_sock( const char * path )
         close( fd );
         return -1;
     }
-    ipc_newcon( &con->ipc, gl_tree );
+    con->ipc = ipc_newcon( gl_tree );
+    if( NULL == con->ipc )
+    {
+        mallocmsg( sizeof *con->ipc );
+        close( fd );
+        free( con );
+    }
     con->infd = fd;
     con->evin = bufferevent_new( fd, canread, didwrite, ohshit, con );
     if( NULL == con->evin )
     {
-        mallocmsg( -1 );
+        errnomsg( "failed to create bufferevent" );
         close( fd );
+        ipc_freecon( con->ipc );
         free( con );
         return -1;
     }
@@ -274,6 +282,7 @@ client_new_cmd( char * const * cmd )
     if( NULL == con->evout )
     {
         bufferevent_free( con->evin );
+        bufferevent_free( con->evout );
         free( con );
         close( tocmd[1] );
         close( fromcmd[0] );
@@ -283,7 +292,18 @@ client_new_cmd( char * const * cmd )
     bufferevent_settimeout( con->evout, SERVER_TIMEOUT, SERVER_TIMEOUT );
     bufferevent_enable( con->evout, EV_READ );
 
-    ipc_newcon( &con->ipc, gl_tree );
+    con->ipc = ipc_newcon( gl_tree );
+    if( NULL == con->ipc )
+    {
+        mallocmsg( sizeof *con->ipc );
+        bufferevent_free( con->evin );
+        bufferevent_free( con->evout );
+        free( con );
+        close( tocmd[1] );
+        close( fromcmd[0] );
+        return -1;
+    }
+
     if( 0 > sendvers( con ) )
     {
         exit( 1 );
@@ -686,7 +706,7 @@ canread( struct bufferevent * evin, void * arg )
         return;
     }
 
-    res = ipc_parse( &con->ipc, buf, len, con );
+    res = ipc_parse( con->ipc, buf, len, con );
     if( 0 > res )
     {
         switch( errno )
@@ -720,7 +740,7 @@ flushreqs( struct con * con )
     benc_val_t       pk, * val;
     struct stritem * jj;
 
-    if( !HASVERS( &con->ipc ) )
+    if( !HASVERS( con->ipc ) )
     {
         return;
     }
@@ -741,7 +761,7 @@ flushreqs( struct con * con )
             case IPC_MSG_STARTALL:
             case IPC_MSG_STOPALL:
             case IPC_MSG_REMOVEALL:
-                buf = ipc_mkempty( &con->ipc, &buflen, req->id, req->tag );
+                buf = ipc_mkempty( con->ipc, &buflen, req->id, req->tag );
                 break;
             case IPC_MSG_ADDMANYFILES:
                 ii = 0;
@@ -749,7 +769,7 @@ flushreqs( struct con * con )
                 {
                     ii++;
                 }
-                val = ipc_initval( &con->ipc, req->id, -1, &pk, TYPE_LIST );
+                val = ipc_initval( con->ipc, req->id, -1, &pk, TYPE_LIST );
                 if( NULL != val && !tr_bencListReserve( val, ii ) )
                 {
                     SLIST_FOREACH( jj, req->strs, next )
@@ -763,7 +783,7 @@ flushreqs( struct con * con )
                 SAFEFREESTRLIST( req->strs );
                 break;
             case IPC_MSG_ADDONEFILE:
-                val = ipc_initval( &con->ipc, req->id, -1, &pk, TYPE_DICT );
+                val = ipc_initval( con->ipc, req->id, -1, &pk, TYPE_DICT );
                 if( NULL != val && !tr_bencDictReserve( val, 1 ) )
                 {
                     tr_bencInitStr( tr_bencDictAdd( val, "data" ),
@@ -778,16 +798,16 @@ flushreqs( struct con * con )
             case IPC_MSG_DOWNLIMIT:
             case IPC_MSG_UPLIMIT:
             case IPC_MSG_PEX:
-                buf = ipc_mkint( &con->ipc, &buflen, req->id, -1, req->num );
+                buf = ipc_mkint( con->ipc, &buflen, req->id, -1, req->num );
                 break;
             case IPC_MSG_DIR:
-                buf = ipc_mkstr( &con->ipc, &buflen, req->id, -1, req->str );
+                buf = ipc_mkstr( con->ipc, &buflen, req->id, -1, req->str );
                 SAFEFREE( req->str );
                 break;
             case IPC_MSG_START:
             case IPC_MSG_STOP:
             case IPC_MSG_REMOVE:
-                val = ipc_initval( &con->ipc, req->id, -1, &pk, TYPE_LIST );
+                val = ipc_initval( con->ipc, req->id, -1, &pk, TYPE_LIST );
                 if( NULL != val && !tr_bencListReserve( val, req->listlen ) )
                 {
                     for( ii = 0; ii < req->listlen; ii++ )
@@ -802,7 +822,7 @@ flushreqs( struct con * con )
                 break;
             case IPC_MSG_GETINFOALL:
             case IPC_MSG_GETSTATALL:
-                buf = ipc_mkgetinfo( &con->ipc, &buflen, req->id, req->tag,
+                buf = ipc_mkgetinfo( con->ipc, &buflen, req->id, req->tag,
                                      req->types, NULL );
                 break;
             default:
@@ -838,7 +858,7 @@ sendvers( struct con * con )
     uint8_t   * buf;
     size_t      len;
 
-    buf = ipc_mkvers( &len );
+    buf = ipc_mkvers( &len, "Transmission remote" LONG_VERSION_STRING );
     if( NULL == buf )
     {
         if( EPERM == errno )
@@ -881,7 +901,7 @@ infomsg( enum ipc_msg msgid, benc_val_t * list, int64_t tag,
         return;
     }
 
-    bzero( &key, sizeof key );
+    memset( &key, 0, sizeof key );
     key.tag = tag;
     resp = RB_FIND( resptree, &gl_resps, &key );
     if( NULL == resp || NULL == resp->infocb )
@@ -933,7 +953,7 @@ statmsg( enum ipc_msg msgid, benc_val_t * list, int64_t tag,
         return;
     }
 
-    bzero( &key, sizeof key );
+    memset( &key, 0, sizeof key );
     key.tag = tag;
     resp = RB_FIND( resptree, &gl_resps, &key );
     if( NULL == resp || NULL == resp->statcb )
@@ -998,7 +1018,7 @@ defmsg( enum ipc_msg msgid, benc_val_t * val, int64_t tag, void * arg UNUSED )
             break;
     }
 
-    bzero( &key, sizeof key );
+    memset( &key, 0, sizeof key );
     key.tag = tag;
     resp = RB_FIND( resptree, &gl_resps, &key );
     if( NULL == resp )
