@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: remote.c 1733 2007-04-17 17:35:04Z joshe $
+ * $Id: remote.c 3401 2007-10-13 17:33:53Z charles $
  *
  * Copyright (c) 2007 Joshua Elsasser
  *
@@ -29,19 +29,22 @@
 #include <ctype.h>
 #include <event.h>
 #include <getopt.h>
+#include <signal.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 
+#include <libtransmission/bsdtree.h>
+#include <libtransmission/ipcparse.h>
+#include <libtransmission/transmission.h>
+#include <libtransmission/trcompat.h>
+
 #include "bsdqueue.h"
-#include "bsdtree.h"
 #include "client.h"
 #include "errors.h"
 #include "misc.h"
-#include "transmission.h"
-#include "trcompat.h"
 
 #define BESTDECIMAL(d)          (10.0 > (d) ? 2 : (100.0 > (d) ? 1 : 0))
 
@@ -50,6 +53,7 @@ struct opts
     int               proxy;
     char           ** proxycmd;
     enum confpathtype type;
+    const char      * sock;
     struct strlist    files;
     int               sendquit;
     int               port;
@@ -73,6 +77,8 @@ struct opts
 struct torinfo
 {
     int     id;
+    int     infogood;
+    int     statgood;
     char  * name;
     int64_t size;
     char  * state;
@@ -126,9 +132,9 @@ static struct torhashes gl_hashids      = RB_INITIALIZER( &gl_hashids );
 static int              gl_gotlistinfo  = 0;
 static int              gl_gotliststat  = 0;
 
-RB_GENERATE_STATIC( torlist,   torinfo, idlinks,   toridcmp );
-RB_GENERATE_STATIC( tornames,  torinfo, namelinks, tornamecmp );
-RB_GENERATE_STATIC( torhashes, torhash, link,      torhashcmp );
+RB_GENERATE_STATIC( torlist,   torinfo, idlinks,   toridcmp )
+RB_GENERATE_STATIC( tornames,  torinfo, namelinks, tornamecmp )
+RB_GENERATE_STATIC( torhashes, torhash, link,      torhashcmp )
 
 int
 main( int argc, char ** argv )
@@ -143,6 +149,8 @@ main( int argc, char ** argv )
         exit( 1 );
     }
 
+    signal( SIGPIPE, SIG_IGN );
+
     evbase = event_init();
     client_init( evbase );
 
@@ -152,8 +160,15 @@ main( int argc, char ** argv )
     }
     else
     {
-        confpath( sockpath, sizeof sockpath, CONF_FILE_SOCKET, o.type );
-        client_new_sock( sockpath );
+        if( NULL == o.sock )
+        {
+            confpath( sockpath, sizeof sockpath, CONF_FILE_SOCKET, o.type );
+            client_new_sock( sockpath );
+        }
+        else
+        {
+            client_new_sock( o.sock );
+        }
     }
 
     if( ( o.sendquit                &&   0 > client_quit     (           ) ) ||
@@ -210,8 +225,8 @@ usage( const char * msg, ... )
     printf(
   "usage: %s [options]\n"
   "\n"
-  "Transmission %s (r%d) http://transmission.m0k.org/\n"
-  "A free, lightweight BitTorrent client with a simple, intuitive interface.\n"
+  "Transmission %s http://transmission.m0k.org/\n"
+  "A fast and easy BitTorrent client\n"
   "\n"
   "  -a --add <torrent>        Add a torrent\n"
   "  -d --download-limit <int> Max download rate in KiB/s\n"
@@ -233,11 +248,12 @@ usage( const char * msg, ... )
   "  -S --stop <hash>          Stop the torrent with the given hash\n"
   "  -S --stop all             Stop all running torrents\n"
   "  -t --type daemon          Use the daemon frontend, transmission-daemon\n"
-  "  -t --type gtk             Use the GTK+ frontend, transmission-gtk\n"
+  "  -t --type gtk             Use the GTK+ frontend, transmission\n"
+  "  -t --type mac             Use the MacOS X frontend\n"
   "  -u --upload-limit <int>   Max upload rate in KiB/s\n"
   "  -U --upload-unlimited     No upload rate limit\n"
   "  -x --proxy                Use proxy command to connect to frontend\n",
-            getmyname(), VERSION_STRING, VERSION_REVISION );
+            getmyname(), LONG_VERSION_STRING );
     exit( 0 );
 }
 
@@ -272,7 +288,7 @@ readargs( int argc, char ** argv, struct opts * opts )
     int opt, gotmsg;
 
     gotmsg = 0;
-    bzero( opts, sizeof *opts );
+    memset( opts, 0, sizeof *opts );
     opts->type = CONF_PATH_TYPE_DAEMON;
     SLIST_INIT( &opts->files );
     opts->map = -1;
@@ -352,14 +368,24 @@ readargs( int argc, char ** argv, struct opts * opts )
                 if( 0 == strcasecmp( "daemon", optarg ) )
                 {
                     opts->type  = CONF_PATH_TYPE_DAEMON;
+                    opts->sock  = NULL;
                 }
                 else if( 0 == strcasecmp( "gtk", optarg ) )
                 {
                     opts->type  = CONF_PATH_TYPE_GTK;
+                    opts->sock  = NULL;
+                }
+                else if( 0 == strcasecmp( "mac", optarg ) ||
+                         0 == strcasecmp( "osx", optarg ) ||
+                         0 == strcasecmp( "macos", optarg ) ||
+                         0 == strcasecmp( "macosx", optarg ) )
+                {
+                    opts->type  = CONF_PATH_TYPE_OSX;
+                    opts->sock  = NULL;
                 }
                 else
                 {
-                    usage( "invalid type: %s", optarg );
+                    opts->sock  = optarg;
                 }
                 break;
             case 'u':
@@ -466,7 +492,7 @@ hasharg( const char * arg, struct strlist * list, int * all )
     }
 
     /* try to look up the hash in the hash tree */
-    bzero( &key, sizeof key );
+    memset( &key, 0, sizeof key );
     strlcpy( key.hash, listitem->str, sizeof key.hash );
     treeitem = RB_FIND( torhashes, &gl_hashids, &key );
     if( NULL == treeitem )
@@ -596,7 +622,7 @@ infomsg( const struct cl_info * cinfo )
         return;
     }
 
-    bzero( &key, sizeof key );
+    memset( &key, 0, sizeof key );
     key.id = cinfo->id;
     tinfo = RB_FIND( torlist, &gl_torinfo, &key );
     if( NULL == tinfo )
@@ -611,6 +637,7 @@ infomsg( const struct cl_info * cinfo )
         RB_INSERT( torlist, &gl_torinfo, tinfo );
     }
 
+    tinfo->infogood = 1;
     free( tinfo->name );
     tinfo->name = strdup_noctrl( cinfo->name );
     tinfo->size = cinfo->size;
@@ -640,7 +667,7 @@ statmsg( const struct cl_stat * st )
         return;
     }
 
-    bzero( &key, sizeof key );
+    memset( &key, 0, sizeof key );
     key.id = st->id;
     info = RB_FIND( torlist, &gl_torinfo, &key );
     if( NULL == info )
@@ -655,6 +682,7 @@ statmsg( const struct cl_stat * st )
         RB_INSERT( torlist, &gl_torinfo, info );
     }
 
+    info->statgood = 1;
     free( info->state );
     info->state     = strdup_noctrl( st->state );
     info->eta       = st->eta;
@@ -690,7 +718,7 @@ hashmsg( const struct cl_info * inf )
         return;
     }
 
-    bzero( &key, sizeof key );
+    memset( &key, 0, sizeof key );
     strlcpy( key.hash, inf->hash, sizeof key.hash );
     found = RB_FIND( torhashes, &gl_hashids, &key );
     if( NULL != found )
@@ -812,11 +840,11 @@ sendidreqs( void )
     struct torhash   key, * found;
 
     ret = -1;
-    bzero( &key, sizeof key );
+    memset( &key, 0, sizeof key );
 
     for( ii = 0; ARRAYLEN( reqs ) > ii; ii++)
     {
-        if( SLIST_EMPTY( reqs[ii].list ) )
+        if( NULL == reqs[ii].list || SLIST_EMPTY( reqs[ii].list ) )
         {
             continue;
         }
@@ -889,6 +917,11 @@ printlisting( void )
         RB_REMOVE( tornames, &names, ii );
         RB_REMOVE( torlist, &gl_torinfo, ii );
 
+        if( !ii->infogood || !ii->statgood )
+        {
+            goto free;
+        }
+
         /* massage some numbers into a better format for printing */
         size      = fmtsize( ii->size, &units );
         upspeed   = fmtsize( ii->rateup, &upunits );
@@ -907,6 +940,12 @@ printlisting( void )
             printf( "%.*f%% checking files",
                     BESTDECIMAL( progress ), progress );
         }
+        /* print progress */
+        else if( 0 == strcasecmp( "waiting to checking", ii->state ) )
+        {
+            printf( "%.*f%% waiting to check files",
+                    BESTDECIMAL( progress ), progress );
+        }
         /* print download progress, speeds, and eta */
         else if( 0 == strcasecmp( "downloading", ii->state ) )
         {
@@ -960,11 +999,15 @@ printlisting( void )
         {
             if( NULL == ii->errorcode )
             {
-                printf( " [error: %s]", ii->errormsg );
+                printf( " [error - %s]", ii->errormsg );
+            }
+            else if( NULL == ii->errormsg )
+            {
+                printf( " [error (%s)]", ii->errorcode );
             }
             else
             {
-                printf( " [%s error]", ii->errorcode );
+                printf( " [error (%s) - %s]", ii->errorcode, ii->errormsg );
             }
         }
 
@@ -972,6 +1015,7 @@ printlisting( void )
         putchar( '\n' );
 
         /* free the stuff for this torrent */
+      free:
         free( ii->name );
         free( ii->state );
         free( ii->errorcode );
