@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: server.c 1806 2007-04-26 07:03:36Z joshe $
+ * $Id: server.c 3579 2007-10-26 04:15:33Z joshe $
  *
  * Copyright (c) 2007 Joshua Elsasser
  *
@@ -36,10 +36,11 @@
 #include <string.h>
 #include <unistd.h>
 
-#include "bsdtree.h"
-#include "bencode.h"
+#include <libtransmission/bsdtree.h>
+#include <libtransmission/bencode.h>
+#include <libtransmission/ipcparse.h>
+
 #include "errors.h"
-#include "ipc.h"
 #include "misc.h"
 #include "server.h"
 #include "torrents.h"
@@ -51,7 +52,7 @@ struct client
 {
     int                  fd;
     struct bufferevent * ev;
-    struct ipc_info      ipc;
+    struct ipc_info    * ipc;
     RB_ENTRY( client )   link;
 };
 
@@ -62,7 +63,7 @@ static void noop     ( struct bufferevent *, void * );
 static void byebye   ( struct bufferevent *, short, void * );
 static void doread   ( struct bufferevent *, void * );
 static int  queuemsg ( struct client *, uint8_t *, size_t );
-static void msgresp  ( struct client *, int64_t, enum ipc_msg );
+static int  msgresp  ( struct client *, int64_t, enum ipc_msg );
 static void defmsg   ( enum ipc_msg, benc_val_t *, int64_t, void * );
 static void noopmsg  ( enum ipc_msg, benc_val_t *, int64_t, void * );
 static void addmsg1  ( enum ipc_msg, benc_val_t *, int64_t, void * );
@@ -178,16 +179,34 @@ server_listen( int fd )
     }
 
     event_set( ev, fd, EV_READ | EV_PERSIST, newclient, ev );
-    /* XXX event_base_set( gl_base, ev ); */
+    event_base_set( gl_base, ev );
     event_add( ev, NULL );
 
     return 0;
 }
 
+void
+server_quit( void )
+{
+    struct client * ii, * next;
+
+    if(gl_exiting)
+        return;
+
+    torrent_exit( 0 );
+    gl_exiting = 1;
+
+    for( ii = RB_MIN( allclients, &gl_clients ); NULL != ii; ii = next )
+    {
+        next = RB_NEXT( allclients, &gl_clients, ii );
+        byebye( ii->ev, EVBUFFER_EOF, NULL );
+    }
+}
+
 void
 newclient( int fd, short event UNUSED, void * arg )
 {
-    struct sockaddr_un   sun;
+    struct sockaddr_un   sa;
     struct client      * client, * old;
     socklen_t            socklen;
     struct bufferevent * clev;
@@ -210,8 +229,8 @@ newclient( int fd, short event UNUSED, void * arg )
             return;
         }
 
-        socklen = sizeof sun;
-        clfd = accept( fd, ( struct sockaddr * )&sun, &socklen );
+        socklen = sizeof sa;
+        clfd = accept( fd, ( struct sockaddr * )&sa, &socklen );
         if( 0 > clfd )
         {
             if( EWOULDBLOCK != errno && EAGAIN != errno &&
@@ -223,19 +242,28 @@ newclient( int fd, short event UNUSED, void * arg )
             break;
         }
 
+        client->ipc = ipc_newcon( gl_tree );
+        if( NULL == client->ipc )
+        {
+            mallocmsg( sizeof *client->ipc );
+            close( clfd );
+            free( client );
+            return;
+        }
+
         clev = bufferevent_new( clfd, doread, noop, byebye, client );
         if( NULL == clev )
         {
+            errnomsg( "failed to create bufferevent" );
             close( clfd );
+            ipc_freecon( client->ipc );
             free( client );
-            mallocmsg( -1 );
             return;
         }
-        /* XXX bufferevent_base_set( gl_base, clev ); */
+        bufferevent_base_set( gl_base, clev );
         bufferevent_settimeout( clev, CLIENT_TIMEOUT, CLIENT_TIMEOUT );
 
         client->fd      = clfd;
-        ipc_newcon( &client->ipc, gl_tree );
         client->ev      = clev;
         old = RB_INSERT( allclients, &gl_clients, client );
         assert( NULL == old );
@@ -246,7 +274,7 @@ newclient( int fd, short event UNUSED, void * arg )
         }
 
         bufferevent_enable( clev, EV_READ );
-        buf = ipc_mkvers( &buflen );
+        buf = ipc_mkvers( &buflen, "Transmission daemon " LONG_VERSION_STRING );
         if( 0 > queuemsg( client, buf, buflen ) )
         {
             free( buf );
@@ -291,13 +319,14 @@ byebye( struct bufferevent * ev, short what, void * arg UNUSED )
         }
     }
 
-    bzero( &key, sizeof key );
+    memset( &key, 0, sizeof key );
     key.ev = ev;
     client = RB_FIND( allclients, &gl_clients, &key );
     assert( NULL != client );
     RB_REMOVE( allclients, &gl_clients, client );
     bufferevent_free( ev );
     close( client->fd );
+    ipc_freecon( client->ipc );
     if( gl_debug )
     {
         printf( "*** client %i went bye-bye\n", client->fd );
@@ -330,7 +359,7 @@ doread( struct bufferevent * ev, void * arg )
         return;
     }
 
-    res = ipc_parse( &client->ipc, buf, len, client );
+    res = ipc_parse( client->ipc, buf, len, client );
 
     if( gl_exiting )
     {
@@ -388,20 +417,23 @@ queuemsg( struct client * client, uint8_t * buf, size_t buflen )
     return 0;
 }
 
-void
+int
 msgresp( struct client * client, int64_t tag, enum ipc_msg id )
 {
     uint8_t * buf;
     size_t    buflen;
+    int       ret;
 
     if( 0 >= tag )
     {
-        return;
+        return 0;
     }
 
-    buf = ipc_mkempty( &client->ipc, &buflen, id, tag );
-    queuemsg( client, buf, buflen );
+    buf = ipc_mkempty( client->ipc, &buflen, id, tag );
+    ret = queuemsg( client, buf, buflen );
     free( buf );
+
+    return ret;
 }
 
 void
@@ -430,15 +462,14 @@ addmsg1( enum ipc_msg id UNUSED, benc_val_t * val, int64_t tag, void * arg )
     int             ii, tor;
     size_t          buflen;
     uint8_t       * buf;
-    tr_info_t     * inf;
 
     if( NULL == val || TYPE_LIST != val->type )
     {
-        msgresp( client, tag, IPC_MSG_NOTSUP );
+        msgresp( client, tag, IPC_MSG_BAD );
         return;
     }
 
-    added = ipc_initval( &client->ipc, IPC_MSG_INFO, tag, &pk, TYPE_LIST );
+    added = ipc_initval( client->ipc, IPC_MSG_INFO, tag, &pk, TYPE_LIST );
     if( NULL == added )
     {
         errnomsg( "failed to build message" );
@@ -457,7 +488,7 @@ addmsg1( enum ipc_msg id UNUSED, benc_val_t * val, int64_t tag, void * arg )
         tor = torrent_add_file( file->val.s.s, NULL, -1 );
         if( TORRENT_ID_VALID( tor ) )
         {
-            inf = torrent_info( tor );
+            const tr_info * inf = torrent_info( tor );
             if( 0 > ipc_addinfo( added, tor, inf, 0 ) )
             {
                 errnomsg( "failed to build message" );
@@ -483,11 +514,10 @@ addmsg2( enum ipc_msg id UNUSED, benc_val_t * dict, int64_t tag, void * arg )
     size_t          buflen;
     uint8_t       * buf;
     const char    * dir;
-    tr_info_t     * inf;
 
     if( NULL == dict || TYPE_DICT != dict->type )
     {
-        msgresp( client, tag, IPC_MSG_NOTSUP );
+        msgresp( client, tag, IPC_MSG_BAD );
         return;
     }
 
@@ -508,7 +538,7 @@ addmsg2( enum ipc_msg id UNUSED, benc_val_t * dict, int64_t tag, void * arg )
         val = tr_bencDictFind( dict, "file" );
         if( NULL == val || TYPE_STR != val->type )
         {
-            msgresp( client, tag, IPC_MSG_NOTSUP );
+            msgresp( client, tag, IPC_MSG_BAD );
             return;
         }
         /* XXX detect duplicates and return a message indicating so */
@@ -517,7 +547,8 @@ addmsg2( enum ipc_msg id UNUSED, benc_val_t * dict, int64_t tag, void * arg )
 
     if( TORRENT_ID_VALID( tor ) )
     {
-        val = ipc_initval( &client->ipc, IPC_MSG_INFO, tag, &pk, TYPE_LIST );
+        const tr_info * inf;
+        val = ipc_initval( client->ipc, IPC_MSG_INFO, tag, &pk, TYPE_LIST );
         if( NULL == val )
         {
             errnomsg( "failed to build message" );
@@ -547,16 +578,7 @@ void
 quitmsg( enum ipc_msg id UNUSED, benc_val_t * val UNUSED, int64_t tag UNUSED,
          void * arg UNUSED )
 {
-    struct client * ii, * next;
-
-    torrent_exit( 0 );
-    gl_exiting = 1;
-
-    for( ii = RB_MIN( allclients, &gl_clients ); NULL != ii; ii = next )
-    {
-        next = RB_NEXT( allclients, &gl_clients, ii );
-        byebye( ii->ev, EVBUFFER_EOF, NULL );
-    }
+    server_quit();
 }
 
 void
@@ -567,7 +589,7 @@ intmsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
 
     if( NULL == val || TYPE_INT != val->type )
     {
-        msgresp( client, tag, IPC_MSG_NOTSUP );
+        msgresp( client, tag, IPC_MSG_BAD );
         return;
     }
 
@@ -607,7 +629,7 @@ strmsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
 
     if( NULL == val || TYPE_STR != val->type )
     {
-        msgresp( client, tag, IPC_MSG_NOTSUP );
+        msgresp( client, tag, IPC_MSG_BAD );
         return;
     }
 
@@ -659,7 +681,7 @@ infomsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
     }
 
     /* initialize packet */
-    pkinf = ipc_initval( &client->ipc, respid, tag, &pk, TYPE_LIST );
+    pkinf = ipc_initval( client->ipc, respid, tag, &pk, TYPE_LIST );
     if( NULL == pkinf )
     {
         errnomsg( "failed to build message" );
@@ -672,7 +694,7 @@ infomsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
     {
         if( NULL == val || TYPE_LIST != val->type )
         {
-            msgresp( client, tag, IPC_MSG_NOTSUP );
+            msgresp( client, tag, IPC_MSG_BAD );
             tr_bencFree( &pk );
             return;
         }
@@ -694,7 +716,7 @@ infomsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
     {
         if( NULL == val || TYPE_DICT != val->type )
         {
-            msgresp( client, tag, IPC_MSG_NOTSUP );
+            msgresp( client, tag, IPC_MSG_BAD );
             tr_bencFree( &pk );
             return;
         }
@@ -703,7 +725,7 @@ infomsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
         if( NULL == typelist || TYPE_LIST != typelist->type ||
             NULL == idlist   || TYPE_LIST != idlist->type )
         {
-            msgresp( client, tag, IPC_MSG_NOTSUP );
+            msgresp( client, tag, IPC_MSG_BAD );
             tr_bencFree( &pk );
             return;
         }
@@ -736,32 +758,15 @@ infomsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
 int
 addinfo( benc_val_t * list, int id, int types )
 {
-    tr_info_t * inf;
-
-    inf = torrent_info( id );
-    if( NULL == inf )
-    {
-        return 0;
-    }
-
-    return ipc_addinfo( list, id, inf, types );
+    const tr_info * inf = torrent_info( id );
+    return inf ? ipc_addinfo( list, id, inf, types ) : 0;
 }
 
 int
 addstat( benc_val_t * list, int id, int types )
 {
-    tr_stat_t * st;
-    tr_info_t * inf;
-
-    st = torrent_stat( id );
-    if( NULL == st )
-    {
-        return 0;
-    }
-    inf = torrent_info( id );
-    assert( NULL != inf );
-
-    return ipc_addstat( list, id, inf, st, types );
+    const tr_stat * st = torrent_stat( id );
+    return st ? ipc_addstat( list, id, st, types ) : 0;
 }
 
 void
@@ -806,6 +811,10 @@ tormsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
         while( NULL != ( iter = torrent_iter( iter, &ii ) ) )
         {
             func( ii );
+            if( torrent_remove == func )
+            {
+                iter = NULL;
+            }
         }
     }
     /* remove/start/stop requested list of torrents */
@@ -813,7 +822,7 @@ tormsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
     {
         if( NULL == val || TYPE_LIST != val->type )
         {
-            msgresp( client, tag, IPC_MSG_NOTSUP );
+            msgresp( client, tag, IPC_MSG_BAD );
             return;
         }
         for( ii = 0; val->val.l.count > ii; ii++ )
@@ -839,15 +848,14 @@ lookmsg( enum ipc_msg id UNUSED, benc_val_t * val, int64_t tag, void * arg )
     int             ii;
     benc_val_t    * hash, pk, * pkinf;
     int64_t         found;
-    tr_info_t     * inf;
 
     if( NULL == val || TYPE_LIST != val->type )
     {
-        msgresp( client, tag, IPC_MSG_NOTSUP );
+        msgresp( client, tag, IPC_MSG_BAD );
         return;
     }
 
-    pkinf = ipc_initval( &client->ipc, IPC_MSG_INFO, tag, &pk, TYPE_LIST );
+    pkinf = ipc_initval( client->ipc, IPC_MSG_INFO, tag, &pk, TYPE_LIST );
     if( NULL == pkinf )
     {
         errnomsg( "failed to build message" );
@@ -857,12 +865,13 @@ lookmsg( enum ipc_msg id UNUSED, benc_val_t * val, int64_t tag, void * arg )
 
     for( ii = 0; val->val.l.count > ii; ii++ )
     {
+        const tr_info * inf;
         hash = &val->val.l.vals[ii];
         if( NULL == hash || TYPE_STR != hash->type ||
             SHA_DIGEST_LENGTH * 2 != hash->val.s.i )
         {
             tr_bencFree( &pk );
-            msgresp( client, tag, IPC_MSG_NOTSUP );
+            msgresp( client, tag, IPC_MSG_BAD );
             return;
         }
         found = torrent_lookup( ( uint8_t * )hash->val.s.s );
@@ -897,31 +906,31 @@ prefmsg( enum ipc_msg id, benc_val_t * val UNUSED, int64_t tag, void * arg )
     switch( id )
     {
         case IPC_MSG_GETAUTOMAP:
-            buf = ipc_mkint( &client->ipc, &buflen, IPC_MSG_AUTOMAP, tag,
+            buf = ipc_mkint( client->ipc, &buflen, IPC_MSG_AUTOMAP, tag,
                              torrent_get_port_mapping() );
             break;
         case IPC_MSG_GETAUTOSTART:
-            buf = ipc_mkint( &client->ipc, &buflen, IPC_MSG_AUTOSTART, tag,
+            buf = ipc_mkint( client->ipc, &buflen, IPC_MSG_AUTOSTART, tag,
                              torrent_get_autostart() );
             break;
         case IPC_MSG_GETDIR:
-            buf = ipc_mkstr( &client->ipc, &buflen, IPC_MSG_DIR, tag,
+            buf = ipc_mkstr( client->ipc, &buflen, IPC_MSG_DIR, tag,
                              torrent_get_directory() );
             break;
         case IPC_MSG_GETDOWNLIMIT:
-            buf = ipc_mkint( &client->ipc, &buflen, IPC_MSG_DOWNLIMIT, tag,
+            buf = ipc_mkint( client->ipc, &buflen, IPC_MSG_DOWNLIMIT, tag,
                              torrent_get_downlimit() );
             break;
         case IPC_MSG_GETPEX:
-            buf = ipc_mkint( &client->ipc, &buflen, IPC_MSG_PEX, tag,
+            buf = ipc_mkint( client->ipc, &buflen, IPC_MSG_PEX, tag,
                              torrent_get_pex() );
             break;
         case IPC_MSG_GETPORT:
-            buf = ipc_mkint( &client->ipc, &buflen, IPC_MSG_PORT, tag,
+            buf = ipc_mkint( client->ipc, &buflen, IPC_MSG_PORT, tag,
                              torrent_get_port() );
             break;
         case IPC_MSG_GETUPLIMIT:
-            buf = ipc_mkint( &client->ipc, &buflen, IPC_MSG_UPLIMIT, tag,
+            buf = ipc_mkint( client->ipc, &buflen, IPC_MSG_UPLIMIT, tag,
                              torrent_get_uplimit() );
             break;
         default:
@@ -945,11 +954,11 @@ supmsg( enum ipc_msg id UNUSED, benc_val_t * val, int64_t tag, void * arg )
 
     if( NULL == val || TYPE_LIST != val->type )
     {
-        msgresp( client, tag, IPC_MSG_NOTSUP );
+        msgresp( client, tag, IPC_MSG_BAD );
         return;
     }
 
-    pkval = ipc_initval( &client->ipc, IPC_MSG_SUP, tag, &pk, TYPE_LIST );
+    pkval = ipc_initval( client->ipc, IPC_MSG_SUP, tag, &pk, TYPE_LIST );
     if( NULL == pkval )
     {
         errnomsg( "failed to build message" );
@@ -971,11 +980,11 @@ supmsg( enum ipc_msg id UNUSED, benc_val_t * val, int64_t tag, void * arg )
         if( NULL == name || TYPE_STR != name->type )
         {
             tr_bencFree( &pk );
-            msgresp( client, tag, IPC_MSG_NOTSUP );
+            msgresp( client, tag, IPC_MSG_BAD );
             return;
         }
-        found = ipc_msgid( &client->ipc, name->val.s.s );
-        if( IPC__MSG_COUNT == found )
+        found = ipc_msgid( client->ipc, name->val.s.s );
+        if( IPC__MSG_COUNT == found || !ipc_ishandled( client->ipc, found ) )
         {
             continue;
         }
