@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: torrents.c 1732 2007-04-17 17:31:02Z joshe $
+ * $Id: torrents.c 3579 2007-10-26 04:15:33Z joshe $
  *
  * Copyright (c) 2007 Joshua Elsasser
  *
@@ -39,13 +39,14 @@
 #include <string.h>
 #include <unistd.h>
 
-#include "bsdtree.h"
-#include "bencode.h"
+#include <libtransmission/bsdtree.h>
+#include <libtransmission/bencode.h>
+#include <libtransmission/transmission.h>
+#include <libtransmission/trcompat.h>
+
 #include "errors.h"
 #include "misc.h"
 #include "torrents.h"
-#include "transmission.h"
-#include "trcompat.h"
 
 #define EXIT_TIMEOUT 10         /* how many seconds to wait on exit */
 #define TIMER_SECS   1          /* timer interval seconds */
@@ -54,9 +55,8 @@
 struct tor
 {
     int             id;
-    int             deleting;
     uint8_t         hash[SHA_DIGEST_LENGTH];
-    tr_torrent_t  * tor;
+    tr_torrent    * tor;
     int             pexset;
     int             pex;
     RB_ENTRY( tor ) idlinks;
@@ -75,12 +75,12 @@ static int          loadstate  ( void );
 static int          savestate  ( void );
 static int          toridcmp   ( struct tor *, struct tor * );
 static int          torhashcmp ( struct tor *, struct tor * );
-static struct tor * idlookup   ( int, int );
-static struct tor * hashlookup ( const uint8_t *, int );
+static struct tor * idlookup   ( int );
+static struct tor * hashlookup ( const uint8_t * );
 static struct tor * iterate    ( struct tor * );
 
 static struct event_base * gl_base      = NULL;
-static tr_handle_t       * gl_handle    = NULL;
+static tr_handle         * gl_handle    = NULL;
 static struct tortree      gl_tree      = RB_INITIALIZER( &gl_tree );
 static struct hashtree     gl_hashes    = RB_INITIALIZER( &gl_hashes );
 static int                 gl_lastid    = 0;
@@ -178,45 +178,42 @@ void
 torrent_start( int id )
 {
     struct tor * tor;
-    tr_stat_t  * st;
 
     assert( NULL != gl_handle );
     assert( !gl_exiting );
 
-    tor = idlookup( id, 0 );
-    if( NULL == tor )
+    tor = idlookup( id );
+    if( tor != NULL )
     {
-        return;
+        const tr_stat  * st = tr_torrentStat( tor->tor );
+
+        if( !TR_STATUS_IS_ACTIVE( st->status ) )
+        {
+            tr_torrentStart( tor->tor );
+            savestate();
+        }
     }
 
-    st = tr_torrentStat( tor->tor );
-    if( TR_STATUS_INACTIVE & st->status )
-    {
-        tr_torrentStart( tor->tor );
-        savestate();
-    }
 }
 
 void
 torrent_stop( int id )
 {
     struct tor * tor;
-    tr_stat_t  * st;
 
     assert( NULL != gl_handle );
     assert( !gl_exiting );
 
-    tor = idlookup( id, 0 );
-    if( NULL == tor )
+    tor = idlookup( id );
+    if( tor != NULL )
     {
-        return;
-    }
+        const tr_stat  * st = tr_torrentStat( tor->tor );
 
-    st = tr_torrentStat( tor->tor );
-    if( TR_STATUS_ACTIVE & st->status )
-    {
-        tr_torrentStop( tor->tor );
-        savestate();
+        if( TR_STATUS_IS_ACTIVE( st->status ) )
+        {
+            tr_torrentStop( tor->tor );
+            savestate();
+        }
     }
 }
 
@@ -228,17 +225,15 @@ torrent_remove( int id )
     assert( NULL != gl_handle );
     assert( !gl_exiting );
 
-    tor = idlookup( id, 0 );
-    if( NULL == tor )
+    tor = idlookup( id );
+    if( tor != NULL )
     {
-        return;
+        closetor( tor, 1 );
+        savestate();
     }
-
-    closetor( tor, 1 );
-    savestate();
 }
 
-tr_info_t *
+const tr_info *
 torrent_info( int id )
 {
     struct tor * tor;
@@ -246,7 +241,7 @@ torrent_info( int id )
     assert( NULL != gl_handle );
     assert( !gl_exiting );
 
-    tor = idlookup( id, 0 );
+    tor = idlookup( id );
     if( NULL == tor )
     {
         return NULL;
@@ -255,7 +250,7 @@ torrent_info( int id )
     return tr_torrentInfo( tor->tor );
 }
 
-tr_stat_t *
+const tr_stat *
 torrent_stat( int id )
 {
     struct tor * tor;
@@ -263,7 +258,7 @@ torrent_stat( int id )
     assert( NULL != gl_handle );
     assert( !gl_exiting );
 
-    tor = idlookup( id, 0 );
+    tor = idlookup( id );
     if( NULL == tor )
     {
         return NULL;
@@ -283,7 +278,7 @@ torrent_lookup( const uint8_t * hashstr )
     assert( NULL != gl_handle );
     assert( !gl_exiting );
 
-    bzero( buf, sizeof buf );
+    memset( buf, 0, sizeof buf );
     for( ii = 0; sizeof( hash ) > ii; ii++ )
     {
         if( !isxdigit( hashstr[2*ii] ) || !isxdigit( hashstr[1+2*ii] ) )
@@ -294,7 +289,7 @@ torrent_lookup( const uint8_t * hashstr )
         hash[ii] = strtol( buf, NULL, 16 );
     }
 
-    tor = hashlookup( hash, 0 );
+    tor = hashlookup( hash );
     if( NULL == tor )
     {
         return -1;
@@ -428,7 +423,8 @@ torrent_set_uplimit( int uplimit )
     assert( NULL != gl_handle );
     assert( !gl_exiting );
     gl_uplimit = uplimit;
-    tr_setGlobalUploadLimit( gl_handle, uplimit );
+    tr_setGlobalSpeedLimit   ( gl_handle, TR_UP, uplimit );
+    tr_setUseGlobalSpeedLimit( gl_handle, TR_UP, uplimit > 0 );
     savestate();
 }
 
@@ -444,7 +440,8 @@ torrent_set_downlimit( int downlimit )
     assert( NULL != gl_handle );
     assert( !gl_exiting );
     gl_downlimit = downlimit;
-    tr_setGlobalDownloadLimit( gl_handle, downlimit );
+    tr_setGlobalSpeedLimit   ( gl_handle, TR_DOWN, downlimit );
+    tr_setUseGlobalSpeedLimit( gl_handle, TR_DOWN, downlimit > 0 );
     savestate();
 }
 
@@ -476,7 +473,7 @@ opentor( const char * path, const char * hash, uint8_t * data, size_t size,
 {
     struct tor * tor, * found;
     int          errcode;
-    tr_info_t  * inf;
+    const tr_info  * inf;
 
     assert( ( NULL != path && NULL == hash && NULL == data ) ||
             ( NULL == path && NULL != hash && NULL == data ) ||
@@ -497,19 +494,20 @@ opentor( const char * path, const char * hash, uint8_t * data, size_t size,
         return NULL;
     }
 
+    if( dir == NULL )
+        dir = gl_dir;
+
     if( NULL != path )
     {
-        tor->tor = tr_torrentInit( gl_handle, path, tor->hash,
-                                   TR_FLAG_SAVE, &errcode );
+        tor->tor = tr_torrentInit( gl_handle, path, dir, 1, &errcode );
     }
     else if( NULL != hash )
     {
-        tor->tor = tr_torrentInitSaved( gl_handle, hash, 0, &errcode );
+        tor->tor = tr_torrentInitSaved( gl_handle, hash, dir, 1, &errcode );
     }
     else
     {
-        tor->tor = tr_torrentInitData( gl_handle, data, size, tor->hash,
-                                       TR_FLAG_SAVE, &errcode );
+        tor->tor = tr_torrentInitData( gl_handle, data, size, dir, 1, &errcode );
     }
 
     if( NULL == tor->tor )
@@ -561,15 +559,12 @@ opentor( const char * path, const char * hash, uint8_t * data, size_t size,
     }
     gl_lastid++;
     tor->id       = gl_lastid;
-    tor->deleting = 0;
 
     assert( sizeof( inf->hash ) == sizeof( tor->hash ) );
     inf = tr_torrentInfo( tor->tor );
     memcpy( tor->hash, inf->hash, sizeof tor->hash );
 
-    tr_torrentSetFolder( tor->tor, ( NULL == dir ? gl_dir : dir ) );
-
-    if( TR_FLAG_PRIVATE & inf->flags )
+    if( inf->isPrivate )
     {
         tor->pexset = 1;
         tor->pex    = 0;
@@ -587,24 +582,24 @@ opentor( const char * path, const char * hash, uint8_t * data, size_t size,
     return tor;
 }
 
+static void
+freetor( struct tor * tor )
+{
+    tr_torrentClose( tor->tor );
+    RB_REMOVE( tortree, &gl_tree, tor );
+    RB_REMOVE( hashtree, &gl_hashes, tor );
+    free( tor );
+}
+
 void
 closetor( struct tor * tor, int calltimer )
 {
-    tr_stat_t  * st;
-
-    if( NULL == tor || tor->deleting )
+    if( NULL != tor )
     {
-        return;
-    }
-    tor->deleting = 1;
+        freetor( tor );
 
-    st = tr_torrentStat( tor->tor );
-    if( TR_STATUS_ACTIVE & st->status )
-    {
-        tr_torrentStop( tor->tor );
+        starttimer( calltimer );
     }
-
-    starttimer( calltimer );
 }
 
 void
@@ -613,7 +608,7 @@ starttimer( int callnow )
     if( !evtimer_initialized( &gl_event ) )
     {
         evtimer_set( &gl_event, timerfunc, NULL );
-        /* XXX event_base_set( gl_base, &gl_event ); */
+        event_base_set( gl_base, &gl_event );
     }
 
     if( callnow )
@@ -622,36 +617,16 @@ starttimer( int callnow )
     }
 }
 
-void
+static void
 timerfunc( int fd UNUSED, short event UNUSED, void * arg UNUSED )
 {
-    struct tor         * tor, * next;
-    tr_handle_status_t * hs;
-    tr_stat_t          * st;
-    int                  stillmore;
-    struct timeval       tv;
+    struct tor       * tor, * next;
+    tr_handle_status * hs;
+    int                stillmore;
+    struct timeval     tv;
 
-    stillmore = 0;
-    for( tor = RB_MIN( tortree, &gl_tree ); NULL != tor; tor = next )
-    {
-        next = RB_NEXT( tortree, &gl_tree, tor );
-        if( !tor->deleting )
-        {
-            continue;
-        }
-        st = tr_torrentStat( tor->tor );
-        if( TR_STATUS_PAUSE & st->status )
-        {
-            tr_torrentClose( gl_handle, tor->tor );
-            RB_REMOVE( tortree, &gl_tree, tor );
-            RB_REMOVE( hashtree, &gl_hashes, tor );
-            free( tor );
-        }
-        else
-        {
-            stillmore = 1;
-        }
-    }
+    /* true if we've still got live torrents... */
+    stillmore = tr_torrentCount( gl_handle ) != 0;
 
     if( gl_exiting )
     {
@@ -673,10 +648,7 @@ timerfunc( int fd UNUSED, short event UNUSED, void * arg UNUSED )
             for( tor = RB_MIN( tortree, &gl_tree ); NULL != tor; tor = next )
             {
                 next = RB_NEXT( tortree, &gl_tree, tor );
-                tr_torrentClose( gl_handle, tor->tor );
-                RB_REMOVE( tortree, &gl_tree, tor );
-                RB_REMOVE( hashtree, &gl_hashes, tor );
-                free( tor );
+                freetor( tor );
             }
             tr_close( gl_handle );
             exit( gl_exitval );
@@ -685,7 +657,7 @@ timerfunc( int fd UNUSED, short event UNUSED, void * arg UNUSED )
 
     if( stillmore )
     {
-        bzero( &tv, sizeof tv );
+        memset( &tv, 0, sizeof tv );
         tv.tv_sec  = TIMER_SECS;
         tv.tv_usec = TIMER_USECS;
         evtimer_add( &gl_event, &tv );
@@ -748,14 +720,16 @@ loadstate( void )
     {
         gl_uplimit = num->val.i;
     }
-    tr_setGlobalUploadLimit( gl_handle, gl_uplimit );
+    tr_setGlobalSpeedLimit( gl_handle, TR_UP, gl_uplimit );
+    tr_setUseGlobalSpeedLimit( gl_handle, TR_UP, gl_uplimit > 0 );
 
     num = tr_bencDictFind( &top, "download-limit" );
     if( NULL != num && TYPE_INT == num->type )
     {
         gl_downlimit = num->val.i;
     }
-    tr_setGlobalDownloadLimit( gl_handle, gl_downlimit );
+    tr_setGlobalSpeedLimit( gl_handle, TR_DOWN, gl_downlimit );
+    tr_setUseGlobalSpeedLimit( gl_handle, TR_DOWN, gl_downlimit > 0 );
 
     str = tr_bencDictFind( &top, "default-directory" );
     if( NULL != str && TYPE_STR == str->type )
@@ -816,8 +790,6 @@ savestate( void )
 {
     benc_val_t   top, * list, * tor;
     struct tor * ii;
-    tr_info_t  * inf;
-    tr_stat_t  * st;
     uint8_t    * buf;
     int          len, pexset;
 
@@ -843,10 +815,7 @@ savestate( void )
     len = 0;
     RB_FOREACH( ii, tortree, &gl_tree )
     {
-        if( !ii->deleting )
-        {
-            len++;
-        }
+        len++;
     }
     if( tr_bencListReserve( list, len ) )
     {
@@ -855,16 +824,14 @@ savestate( void )
 
     RB_FOREACH( ii, tortree, &gl_tree )
     {
-        if( ii->deleting )
-        {
-            continue;
-        }
+        const tr_info * inf;
+        const tr_stat * st;
         tor = tr_bencListAdd( list );
         assert( NULL != tor );
         tr_bencInit( tor, TYPE_DICT );
         inf    = tr_torrentInfo( ii->tor );
         st     = tr_torrentStat( ii->tor );
-        pexset = ( ii->pexset && !( TR_FLAG_PRIVATE & inf->flags ) );
+        pexset = ( ii->pexset && !inf->isPrivate );
         if( tr_bencDictReserve( tor, ( pexset ? 4 : 3 ) ) )
         {
             goto nomem;
@@ -872,7 +839,7 @@ savestate( void )
         tr_bencInitStr( tr_bencDictAdd( tor, "hash" ),
                         inf->hashString, 2 * SHA_DIGEST_LENGTH, 1 );
         tr_bencInitInt( tr_bencDictAdd( tor, "paused" ),
-                        ( TR_STATUS_INACTIVE & st->status ? 1 : 0 ) );
+                        !TR_STATUS_IS_ACTIVE( st->status ) );
         tr_bencInitStr( tr_bencDictAdd( tor, "directory" ),
                         tr_torrentGetFolder( ii->tor ), -1, 1 );
         if( pexset )
@@ -913,33 +880,25 @@ torhashcmp( struct tor * left, struct tor * right )
 }
 
 struct tor *
-idlookup( int id, int wantdel )
+idlookup( int id )
 {
     struct tor key, * found;
 
-    bzero( &key, sizeof key );
+    memset( &key, 0, sizeof key );
     key.id = id;
     found = RB_FIND( tortree, &gl_tree, &key );
-    if( NULL != found && !wantdel && found->deleting )
-    {
-        found = NULL;
-    }
 
     return found;
 }
 
 struct tor *
-hashlookup( const uint8_t * hash, int wantdel )
+hashlookup( const uint8_t * hash )
 {
     struct tor key, * found;
 
-    bzero( &key, sizeof key );
+    memset( &key, 0, sizeof key );
     memcpy( key.hash, hash, sizeof key.hash );
     found = RB_FIND( hashtree, &gl_hashes, &key );
-    if( NULL != found && !wantdel && found->deleting )
-    {
-        found = NULL;
-    }
 
     return found;
 }
@@ -959,10 +918,5 @@ iterate( struct tor * tor )
         tor = RB_NEXT( tortree, &gl_tree, tor );
     }
 
-    while( NULL != tor && tor->deleting )
-    {
-        tor = RB_NEXT( tortree, &gl_tree, tor );
-    }
-
     return tor;
 }
