@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ipc.h 1504 2007-02-19 22:09:05Z joshe $
+ * $Id: ipc.h 1922 2007-05-24 03:29:23Z joshe $
  *
  * Copyright (c) 2006-2007 Transmission authors and contributors
  *
@@ -25,11 +25,12 @@
 #ifndef TG_IPC_H
 #define TG_IPC_H
 
-#include "util.h"
+#include <gtk/gtk.h>
+
+#include "tr_core.h"
 
 void
-ipc_socket_setup( void * wind, add_torrents_func_t addfunc,
-                  callbackfunc_t quitfunc, void * cbdata );
+ipc_socket_setup( GtkWindow * wind, TrCore * core );
 
 gboolean
 ipc_sendfiles_blocking( GList * files );
