@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: clients.c 1811 2007-04-30 01:29:00Z livings124 $
+ * $Id: clients.c 3463 2007-10-19 00:19:16Z charles $
  *
  * Copyright (c) 2005 Transmission authors and contributors
  *
@@ -22,22 +22,27 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
-#include "transmission.h"
+#include <ctype.h> /* isprint */
+#include <stdio.h>
+#include <string.h>
 
-static int charToInt( char character );
+#include "transmission.h"
+#include "trcompat.h"
 
 static int charToInt( char character )
 {
     int value;
     if( character >= 'A' && character <= 'Z' )
         value = 10 + character - 'A';
+    else if( character >= 'a' && character <= 'z')
+        value = 36 + character - 'a';
     else
         value = character - '0';
     
     return value;
 }
 
-char * tr_clientForId( uint8_t * id )
+char * tr_clientForId( const uint8_t * id )
 {
     char * ret = NULL;
     
@@ -46,9 +51,18 @@ char * tr_clientForId( uint8_t * id )
     {
         if( !memcmp( &id[1], "TR", 2 ) )
         {
-            asprintf( &ret, "Transmission %d.%d",
-                      charToInt( id[3] ) * 10 + charToInt( id[4] ),
-                      charToInt( id[5] ) * 10 + charToInt( id[6] ) );
+            /* support old-style Transmission id without maintenance number */
+            if ( !memcmp( &id[3], "00", 2 ) )
+            {
+                asprintf( &ret, "Transmission 0.%d",
+                        charToInt( id[5] ) * 10 + charToInt( id[6] ) );
+            }
+            else
+            {
+                asprintf( &ret, "Transmission %d.%c%c%s",
+                        charToInt( id[3] ), id[4], id[5],
+                        id[6] == 'Z' ? "+" : "" );
+            }
         }
         else if( !memcmp( &id[1], "AZ", 2 ) )
         {
@@ -94,9 +108,8 @@ char * tr_clientForId( uint8_t * id )
         }
         else if( !memcmp( &id[1], "XX", 2 ) )
         {
-            asprintf( &ret, "Xtorrent (%d)",
-                      charToInt( id[3] ) * 1000 + charToInt( id[4] ) * 100
-                      + charToInt( id[5] ) * 10 + charToInt( id[6] ) );
+            asprintf( &ret, "Xtorrent %c.%c (%d)",
+                      id[3], id[4], charToInt( id[5] ) * 10 + charToInt( id[6] ) );
         }
         else if( !memcmp( &id[1], "TS", 2 ) )
         {
@@ -179,6 +192,12 @@ char * tr_clientForId( uint8_t * id )
                       charToInt( id[3] ), charToInt( id[4] ),
                       charToInt( id[5] ) );
         }
+        else if( !memcmp( &id[1], "HL", 2 ) )
+        {
+            asprintf( &ret, "Halite %d.%d.%d",
+                      charToInt( id[3] ), charToInt( id[4] ),
+                      charToInt( id[5] ) );
+        }
         else if( !memcmp( &id[1], "AR", 2 ) )
         {
             asprintf( &ret, "Arctic Torrent" );
@@ -211,6 +230,20 @@ char * tr_clientForId( uint8_t * id )
             asprintf( &ret, "FoxTorrent (%c%c%c%c)",
                       id[3], id[4], id[5], id[6] );
         }
+        else if( !memcmp( &id[1], "GR", 2 ) )
+        {
+            asprintf( &ret, "GetRight %c.%c.%c.%c",
+                      id[3], id[4], id[5], id[6] );
+        }
+        else if( !memcmp( &id[1], "PD", 2 ) )
+        {
+            asprintf( &ret, "Pando %c.%c.%c.%c",
+                      id[3], id[4], id[5], id[6] );
+        }
+        else if( !memcmp( &id[1], "LW", 2 ) )
+        {
+            asprintf( &ret, "LimeWire" );
+        }
         
         if( ret )
         {
@@ -308,6 +341,11 @@ char * tr_clientForId( uint8_t * id )
                   charToInt( id[3] ) * 10 + charToInt( id[4] ),
                       id[5], id[6] );
     }
+    else if( !memcmp( id, "DNA", 3 ) )
+    {
+        asprintf( &ret, "BitTorrent DNA %d.%d.%d", charToInt( id[3] ) * 10 + charToInt( id[4] ),
+                    charToInt( id[5] ) * 10 + charToInt( id[6] ), charToInt( id[7] ) * 10 + charToInt( id[8] ) );
+    }
     else if( !memcmp( id, "Plus", 4 ) )
     {
         asprintf( &ret, "Plus! v2 %c.%c%c", id[4], id[5], id[6] );
@@ -315,7 +353,7 @@ char * tr_clientForId( uint8_t * id )
     else if( !memcmp( id, "XBT", 3 ) )
     {
         asprintf( &ret, "XBT Client %c%c%c%s", id[3], id[4], id[5],
-                  id[6] == 'd' ? " (debug)" : "" );
+                    id[6] == 'd' ? " (debug)" : "" );
     }
     else if( !memcmp( id, "Mbrst", 5 ) )
     {
@@ -325,9 +363,22 @@ char * tr_clientForId( uint8_t * id )
     {
         asprintf( &ret, "BT Protocol Daemon %c%c%c", id[5], id[6], id[7] );
     }
+    else if( id[0] == 'Q' && !memcmp( &id[4], "--", 2 ) )
+    {
+        asprintf( &ret, "BTQueue %d.%d.%d", charToInt( id[1] ),
+                    charToInt( id[2] ), charToInt( id[3] ) );
+    }
+    else if( !memcmp( id, "BLZ", 3 ) )
+    {
+        asprintf( &ret, "Blizzard Downloader %d.%d", id[3] + 1, id[4] );
+    }
+    else if( !memcmp( id, "-WT-", 4 ) )
+    {
+        asprintf( &ret, "BitLet" );
+    }
     else if( !memcmp( id, "LIME", 4 ) )
     {
-        asprintf( &ret, "Limewire" );
+        asprintf( &ret, "LimeWire" );
     }
     else if( !memcmp( id, "-G3", 3 ) )
     {
@@ -345,6 +396,10 @@ char * tr_clientForId( uint8_t * id )
     {
         asprintf( &ret, "eXeem" );
     }
+    else if( '\0' == id[0] && !memcmp( &id[1], "BS", 2 ) )
+    {
+        asprintf( &ret, "BitSpirit %u", ( id[1] == 0 ? 1 : id[1] ) );
+    }
 
     /* No match */
     if( !ret )
