@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: http.c 1773 2007-04-22 15:05:14Z livings124 $
+ * $Id: http.c 3105 2007-09-20 16:32:01Z livings124 $
  *
  * Copyright (c) 2006-2007 Transmission authors and contributors
  *
@@ -22,7 +22,26 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
+#include <assert.h>
+#include <ctype.h>
+#include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include <sys/types.h>
+
+#include "evdns.h"
+
+#ifdef __BEOS__
+extern int vasprintf( char **, const char *, va_list );
+#endif
+
 #include "transmission.h"
+#include "http.h"
+#include "net.h"
+#include "trcompat.h"
+#include "utils.h"
 
 #define HTTP_PORT               80      /* default http port 80 */
 #define HTTP_TIMEOUT            60000   /* one minute http timeout */
@@ -67,7 +86,6 @@ struct tr_http_s {
 #define HTTP_LENGTH_FIXED       3
 #define HTTP_LENGTH_CHUNKED     4
     char           lengthtype;
-    tr_resolve_t * resolve;
     char         * host;
     int            port;
     int            sock;
@@ -135,7 +153,7 @@ tr_httpRequestType( const char * data, int len, char ** method, char ** uri )
     /* copy the method */
     if( 0 <= ret && NULL != method )
     {
-        *method = tr_dupstr( words[0], words[1] - words[0] );
+        *method = tr_strndup( words[0], words[1] - words[0] );
         if( NULL == *method )
         {
             ret = -1;
@@ -144,7 +162,7 @@ tr_httpRequestType( const char * data, int len, char ** method, char ** uri )
     /* copy uri */
     if( 0 <= ret && NULL != uri )
     {
-        *uri = tr_dupstr( words[2], words[3] - words[2] );
+        *uri = tr_strndup( words[2], words[3] - words[2] );
         if( NULL == *uri )
         {
             free( *method );
@@ -328,12 +346,18 @@ tr_httpParseUrl( const char * url, int len,
     const char * pathstart, * hostend;
     int          ii, colon, portnum;
     char         str[6];
-
+    
     if( 0 > len )
     {
         len = strlen( url );
     }
 
+    while( len && url && isspace(*url) )
+    {
+        --len;
+        ++url;
+    }
+
     ii = tr_httpIsUrl( url, len );
     if( 0 >= ii )
     {
@@ -371,7 +395,7 @@ tr_httpParseUrl( const char * url, int len,
 
     if( NULL != host )
     {
-        *host = tr_dupstr( url, hostend - url );
+        *host = tr_strndup( url, hostend - url );
     }
     if( NULL != port )
     {
@@ -381,7 +405,7 @@ tr_httpParseUrl( const char * url, int len,
     {
         if( 0 < len - ( pathstart - url ) )
         {
-            *path = tr_dupstr( pathstart, len - ( pathstart - url ) );
+            *path = tr_strndup( pathstart, len - ( pathstart - url ) );
         }
         else
         {
@@ -449,9 +473,9 @@ tr_httpClient( int method, const char * host, int port, const char * fmt, ... )
 
     if( tr_sprintf( EXPANDBUF( http->header ), " HTTP/1.1" CR LF
                     "Host: %s:%d" CR LF
-                    "User-Agent: %s/%d.%d%d" CR LF
+                    "User-Agent: " TR_NAME "/" LONG_VERSION_STRING CR LF
                     "Connection: close" CR LF,
-                    http->host, http->port, TR_NAME, VERSION_MAJOR, VERSION_MINOR, VERSION_MAINTENANCE ) )
+                    http->host, http->port ) )
     {
         goto err;
     }
@@ -530,6 +554,34 @@ tr_httpAddBody( tr_http_t * http , const char * fmt , ... )
     }
 }
 
+void
+tr_httpGetHeaders( tr_http_t * http, const char ** buf, int * len )
+{
+    *buf = http->header.buf;
+    *len = http->header.used;
+}
+
+void
+tr_httpGetBody( tr_http_t * http, const char ** buf, int * len )
+{
+    *buf = http->body.buf;
+    *len = http->body.used;
+}
+
+static void
+resolve_cb( int result, char type, int count, int ttl, void *addresses, void *arg)
+{
+    tr_http_t * http = (tr_http_t *) arg;
+
+    if( (result!=DNS_ERR_NONE) || (type!=DNS_IPv4_A) || (ttl<0) || (count<1) )
+        http->state = HTTP_STATE_ERROR;
+    else {
+         struct in_addr *in_addrs = addresses;
+         http->sock = tr_netOpenTCP( &in_addrs[0], htons(http->port), 1 );
+         http->state = HTTP_STATE_CONNECT;
+    }
+}
+
 tr_tristate_t
 tr_httpPulse( tr_http_t * http, const char ** data, int * len )
 {
@@ -545,12 +597,11 @@ tr_httpPulse( tr_http_t * http, const char ** data, int * len )
             }
             if( !tr_netResolve( http->host, &addr ) )
             {
-                http->sock = tr_netOpenTCP( addr, htons( http->port ), 1 );
+                http->sock = tr_netOpenTCP( &addr, htons( http->port ), 1 );
                 http->state = HTTP_STATE_CONNECT;
                 break;
             }
-            http->resolve = tr_netResolveInit( http->host );
-            if( NULL == http->resolve )
+            if( evdns_resolve_ipv4( http->host, 0, resolve_cb, http ) )
             {
                 goto err;
             }
@@ -558,19 +609,7 @@ tr_httpPulse( tr_http_t * http, const char ** data, int * len )
             /* fallthrough */
 
         case HTTP_STATE_RESOLVE:
-            switch( tr_netResolvePulse( http->resolve, &addr ) )
-            {
-                case TR_NET_WAIT:
-                    return TR_NET_WAIT;
-                case TR_NET_ERROR:
-                    goto err;
-                case TR_NET_OK:
-                    tr_netResolveClose( http->resolve );
-                    http->resolve = NULL;
-                    http->sock = tr_netOpenTCP( addr, htons( http->port ), 1 );
-                    http->state = HTTP_STATE_CONNECT;
-            }
-            /* fallthrough */
+            return TR_NET_WAIT;
 
         case HTTP_STATE_CONNECT:
             switch( sendrequest( http ) )
@@ -626,12 +665,9 @@ static tr_tristate_t
 sendrequest( tr_http_t * http )
 {
     struct buf * buf;
-    int          ret;
 
-    if( 0 == http->date )
-    {
-        http->date = tr_date();
-    }
+    if( !http->date )
+         http->date = tr_date();
 
     if( 0 > http->sock || tr_date() > http->date + HTTP_TIMEOUT )
     {
@@ -641,15 +677,9 @@ sendrequest( tr_http_t * http )
     buf = ( 0 < http->header.used ? &http->header : &http->body );
     while( 0 < buf->used )
     {
-      ret = tr_netSend( http->sock, (uint8_t *) buf->buf, buf->used );
-        if( ret & TR_NET_CLOSE )
-        {
-            return TR_NET_ERROR;
-        }
-        else if( ret & TR_NET_BLOCK )
-        {
-            return TR_NET_WAIT;
-        }
+        const int ret = tr_netSend( http->sock, buf->buf, buf->used );
+        if( ret & TR_NET_CLOSE ) return TR_NET_ERROR;
+        if( ret & TR_NET_BLOCK ) return TR_NET_WAIT;
         buf->used = 0;
         buf = &http->body;
     }
@@ -829,7 +859,7 @@ learnlength( tr_http_t * http )
         {
             http->lengthtype = HTTP_LENGTH_FIXED;
             http->chunkoff = body - http->header.buf;
-            duped = tr_dupstr( hdr[0].data, hdr[0].len );
+            duped = tr_strndup( hdr[0].data, hdr[0].len );
             http->chunklen = strtol( duped, NULL, 10 );
             free( duped );
         }
@@ -869,15 +899,8 @@ tr_httpWhatsMyAddress( tr_http_t * http )
 void
 tr_httpClose( tr_http_t * http )
 {
-    if( NULL != http->resolve )
-    {
-        tr_netResolveClose( http->resolve );
-    }
     free( http->host );
-    if( 0 <= http->sock )
-    {
-        tr_netClose( http->sock );
-    }
+    tr_netClose( http->sock );
     free( http->header.buf );
     free( http->body.buf );
     free( http );
