@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: metainfo.h 1636 2007-04-02 20:38:23Z joshe $
+ * $Id: metainfo.h 3105 2007-09-20 16:32:01Z livings124 $
  *
  * Copyright (c) 2005 Transmission authors and contributors
  *
@@ -27,12 +27,12 @@
 
 #include "transmission.h"
 
-int tr_metainfoParseFile( tr_info_t *, const char * tag,
+int tr_metainfoParseFile( tr_info *, const char * tag,
                           const char * path, int save );
-int tr_metainfoParseData( tr_info_t *, const char * tag,
-                          uint8_t * data, size_t size, int save );
-int tr_metainfoParseHash( tr_info_t *, const char * tag, const char * hash );
-void tr_metainfoFree( tr_info_t * inf );
+int tr_metainfoParseData( tr_info *, const char * tag,
+                          const uint8_t * data, size_t size, int save );
+int tr_metainfoParseHash( tr_info *, const char * tag, const char * hash );
+void tr_metainfoFree( tr_info * inf );
 void tr_metainfoRemoveSaved( const char * hashString, const char * tag );
 
 #endif
