@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ratecontrol.h 1320 2007-01-08 21:53:55Z livings124 $
+ * $Id: ratecontrol.h 3614 2007-10-28 15:20:23Z charles $
  *
  * Copyright (c) 2006 Transmission authors and contributors
  *
@@ -22,13 +22,19 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
-typedef struct tr_ratecontrol_s tr_ratecontrol_t;
+#ifndef _TR_RATECONTROL_H_
+#define _TR_RATECONTROL_H_
 
-tr_ratecontrol_t * tr_rcInit();
-int                tr_rcCanGlobalTransfer( tr_handle_t * h, int isUpload );
-void               tr_rcSetLimit( tr_ratecontrol_t *, int );
-int                tr_rcCanTransfer( tr_ratecontrol_t * );
-void               tr_rcTransferred( tr_ratecontrol_t *, int );
-float              tr_rcRate( tr_ratecontrol_t * );
-void               tr_rcReset( tr_ratecontrol_t * );
-void               tr_rcClose( tr_ratecontrol_t * );
+typedef struct tr_ratecontrol tr_ratecontrol;
+
+tr_ratecontrol * tr_rcInit( void );
+void             tr_rcSetLimit( tr_ratecontrol *, int );
+int              tr_rcGetLimit( const tr_ratecontrol * );
+int              tr_rcCanTransfer( const tr_ratecontrol * );
+size_t           tr_rcBytesLeft( const tr_ratecontrol * );
+void             tr_rcTransferred( tr_ratecontrol *, size_t byteCount );
+float            tr_rcRate( const tr_ratecontrol * );
+void             tr_rcReset( tr_ratecontrol * );
+void             tr_rcClose( tr_ratecontrol * );
+
+#endif
