@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: upnp.c 1762 2007-04-20 01:27:58Z joshe $
+ * $Id: upnp.c 3553 2007-10-25 14:00:05Z charles $
  *
  * Copyright (c) 2006-2007 Transmission authors and contributors
  *
@@ -22,7 +22,37 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
+#include <assert.h>
+#include <errno.h>
+#include <limits.h>
+#include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <time.h>
+
+#include <arpa/inet.h> /* inet_addr */
+
+#ifdef __BEOS__
+    #include <netdb.h>
+#endif
+
+#include <sys/types.h>
+
 #include "transmission.h"
+#include "http.h"
+#include "net.h"
+#include "trcompat.h"
+#include "upnp.h"
+#include "utils.h"
+#include "xml.h"
+
+/* uncomment this to log requests and responses to ~/transmission-upnp.log */
+/* #define VERBOSE_LOG */
+
+#ifdef VERBOSE_LOG
+#include "platform.h"
+#endif
 
 #define SSDP_ADDR               "239.255.255.250"
 #define SSDP_PORT               1900
@@ -43,14 +73,14 @@
 #define IGD_ADD_CONFLICT        718
 #define IGD_NO_DYNAMIC_MAPPING  725
 
-typedef struct tr_upnp_action_s
+struct upnp_act
 {
     char * name;
     int    len;
     struct { char * name; char * var; char dir; } * args;
-} tr_upnp_action_t;
+};
 
-typedef struct tr_upnp_device_s
+struct upnp_dev
 {
     char                    * id;
     char                    * host;
@@ -75,11 +105,11 @@ typedef struct tr_upnp_device_s
     uint64_t                  lastcheck;
     unsigned int              soapretry : 1;
     tr_http_t               * http;
-    tr_upnp_action_t          getcmd;
-    tr_upnp_action_t          addcmd;
-    tr_upnp_action_t          delcmd;
-    struct tr_upnp_device_s * next;
-} tr_upnp_device_t;
+    struct upnp_act           getcmd;
+    struct upnp_act           addcmd;
+    struct upnp_act           delcmd;
+    struct upnp_dev         * next;
+};
 
 struct tr_upnp_s
 {
@@ -90,72 +120,76 @@ struct tr_upnp_s
     uint64_t           lastdelay;
     unsigned int       active : 1;
     unsigned int       discovering : 1;
-    tr_upnp_device_t * devices;
+    struct upnp_dev  * devices;
 };
 
 static int
 sendSSDP( int fd );
 static int
-mcastStart();
+mcastStart( void );
 static void
 killSock( int * sock );
 static void
 killHttp( tr_http_t ** http );
 static int
-watchSSDP( tr_upnp_device_t ** devices, int fd );
+watchSSDP( struct upnp_dev ** devices, int fd );
 static tr_tristate_t
 recvSSDP( int fd, char * buf, int * len );
 static int
 parseSSDP( char * buf, int len, tr_http_header_t * headers );
 static void
-deviceAdd( tr_upnp_device_t ** first, const char * id, int idLen,
+deviceAdd( struct upnp_dev ** first, const char * id, int idLen,
            const char * url, int urlLen );
 static void
-deviceRemove( tr_upnp_device_t ** prevptr );
+deviceRemove( struct upnp_dev ** prevptr );
 static int
-deviceStop( tr_upnp_device_t * dev );
+deviceStop( struct upnp_dev * dev );
 static int
-devicePulse( tr_upnp_device_t * dev, int port );
+devicePulse( struct upnp_dev * dev, int port );
 static int
-devicePulseHttp( tr_upnp_device_t * dev,
+devicePulseHttp( struct upnp_dev * dev,
                  const char ** body, int * len );
 static tr_http_t *
-devicePulseGetHttp( tr_upnp_device_t * dev );
+devicePulseGetHttp( struct upnp_dev * dev );
 static int
 parseRoot( const char * root, const char *buf, int len,
            char ** soap, char ** scpd, int * ppp );
 static void
 addUrlbase( const char * base, char ** path );
 static int
-parseScpd( const char *buf, int len, tr_upnp_action_t * getcmd,
-           tr_upnp_action_t * addcmd, tr_upnp_action_t * delcmd );
+parseScpd( const char *buf, int len, struct upnp_act * getcmd,
+           struct upnp_act * addcmd, struct upnp_act * delcmd );
 static int
 parseScpdArgs( const char * buf, const char * end,
-               tr_upnp_action_t * action, char dir );
+               struct upnp_act * action, char dir );
 static int
-parseMapping( tr_upnp_device_t * dev, const char * buf, int len );
+parseMapping( struct upnp_dev * dev, const char * buf, int len );
 static char *
 joinstrs( const char *, const char *, const char * );
 static tr_http_t *
 soapRequest( int retry, const char * host, int port, const char * path,
-             const char * type, tr_upnp_action_t * action, ... );
+             const char * type, struct upnp_act * action, ... );
 static void
-actionSetup( tr_upnp_action_t * action, const char * name, int prealloc );
+actionSetup( struct upnp_act * action, const char * name, int prealloc );
 static void
-actionFree( tr_upnp_action_t * action );
+actionFree( struct upnp_act * action );
 static int
-actionAdd( tr_upnp_action_t * action, char * name, char * var,
+actionAdd( struct upnp_act * action, char * name, char * var,
                       char dir );
 #define actionLookupVar( act, nam, len, dir ) \
     ( actionLookup( (act), (nam), (len), (dir), 0 ) )
 #define actionLookupName( act, var, len, dir ) \
     ( actionLookup( (act), (var), (len), (dir), 1 ) )
 static const char *
-actionLookup( tr_upnp_action_t * action, const char * key, int len,
+actionLookup( struct upnp_act * action, const char * key, int len,
               char dir, int getname );
 
+#ifdef VERBOSE_LOG
+static FILE * vlog = NULL;
+#endif 
+
 tr_upnp_t *
-tr_upnpInit()
+tr_upnpInit( void )
 {
     tr_upnp_t * upnp;
 
@@ -168,6 +202,20 @@ tr_upnpInit()
     upnp->infd     = -1;
     upnp->outfd    = -1;
 
+#ifdef VERBOSE_LOG
+    if( NULL == vlog )
+    {
+        char path[MAX_PATH_LENGTH];
+        time_t stupid_api;
+        snprintf( path, sizeof path, "%s/transmission-upnp.log",
+                  tr_getHomeDirectory());
+        vlog = fopen( path, "a" );
+        stupid_api = time( NULL );
+        fprintf( vlog, "opened log at %s\n\n", ctime( &stupid_api ) );
+        fflush( vlog );
+    }
+#endif 
+
     return upnp;
 }
 
@@ -200,8 +248,8 @@ tr_upnpStop( tr_upnp_t * upnp )
 int
 tr_upnpStatus( tr_upnp_t * upnp )
 {
-    tr_upnp_device_t * ii;
-    int                ret;
+    struct upnp_dev * ii;
+    int               ret;
 
     if( !upnp->active )
     {
@@ -257,12 +305,19 @@ tr_upnpClose( tr_upnp_t * upnp )
     }
 
     free( upnp );
+
+#ifdef VERBOSE_LOG
+    if( NULL != vlog )
+    {
+        fclose( vlog );
+    }
+#endif
 }
 
 void
 tr_upnpPulse( tr_upnp_t * upnp )
 {
-    tr_upnp_device_t ** ii;
+    struct upnp_dev ** ii;
 
     if( upnp->active )
     {
@@ -346,13 +401,20 @@ sendSSDP( int fd )
     sin.sin_addr.s_addr = inet_addr( SSDP_ADDR );
     sin.sin_port        = htons( SSDP_PORT );
 
+#ifdef VERBOSE_LOG
+    fprintf( vlog, "send ssdp message, %i bytes:\n", len );
+    fwrite( buf, 1, len, vlog );
+    fputs( "\n\n", vlog );
+    fflush( vlog );
+#endif 
+
     if( 0 > sendto( fd, buf, len, 0,
                     (struct sockaddr*) &sin, sizeof( sin ) ) )
     {
-        if( EAGAIN != errno )
+        if( EAGAIN != sockerrno )
         {
             tr_err( "Could not send SSDP discover message (%s)",
-                    strerror( errno ) );
+                    strerror( sockerrno ) );
         }
         killSock( &fd );
         return -1;
@@ -368,7 +430,7 @@ mcastStart()
     struct in_addr addr;
 
     addr.s_addr = inet_addr( SSDP_ADDR );
-    fd = tr_netMcastOpen( SSDP_PORT, addr );
+    fd = tr_netMcastOpen( SSDP_PORT, &addr );
     if( 0 > fd )
     {
         return -1;
@@ -395,7 +457,7 @@ killHttp( tr_http_t ** http )
 }
 
 static int
-watchSSDP( tr_upnp_device_t ** devices, int fd )
+watchSSDP( struct upnp_dev ** devices, int fd )
 {
     /* XXX what if we get a huge SSDP packet? */
     char buf[512];
@@ -456,11 +518,17 @@ recvSSDP( int fd, char * buf, int * len )
     }
     else if( TR_NET_CLOSE & *len )
     {
-        tr_err( "Could not receive SSDP message (%s)", strerror( errno ) );
+        tr_err( "Could not receive SSDP message (%s)", strerror( sockerrno ) );
         return TR_NET_ERROR;
     }
     else
     {
+#ifdef VERBOSE_LOG
+        fprintf( vlog, "receive ssdp message, %i bytes:\n", *len );
+        fwrite( buf, 1, *len, vlog );
+        fputs( "\n\n", vlog );
+        fflush( vlog );
+#endif 
         return TR_NET_OK;
     }
 }
@@ -522,10 +590,10 @@ parseSSDP( char * buf, int len, tr_http_header_t * hdr )
 }
 
 static void
-deviceAdd( tr_upnp_device_t ** first, const char * id, int idLen,
+deviceAdd( struct upnp_dev ** first, const char * id, int idLen,
            const char * url, int urlLen )
 {
-    tr_upnp_device_t * ii;
+    struct upnp_dev * ii;
 
     for( ii = *first; NULL != ii; ii = ii->next )
     {
@@ -549,7 +617,7 @@ deviceAdd( tr_upnp_device_t ** first, const char * id, int idLen,
         free( ii );
         return;
     }
-    ii->id = tr_dupstr( id, idLen );
+    ii->id = tr_strndup( id, idLen );
     ii->state = UPNPDEV_STATE_ROOT;
     actionSetup( &ii->getcmd, "GetSpecificPortMappingEntry", 8 );
     actionSetup( &ii->addcmd, "AddPortMapping", 8 );
@@ -562,9 +630,9 @@ deviceAdd( tr_upnp_device_t ** first, const char * id, int idLen,
 }
 
 static void
-deviceRemove( tr_upnp_device_t ** prevptr )
+deviceRemove( struct upnp_dev ** prevptr )
 {
-    tr_upnp_device_t * dead;
+    struct upnp_dev * dead;
 
     dead = *prevptr;
     *prevptr = dead->next;
@@ -588,7 +656,7 @@ deviceRemove( tr_upnp_device_t ** prevptr )
 }
 
 static int
-deviceStop( tr_upnp_device_t * dev )
+deviceStop( struct upnp_dev * dev )
 {
     switch( dev->state )
     {
@@ -606,7 +674,7 @@ deviceStop( tr_upnp_device_t * dev )
 }
 
 static int
-devicePulse( tr_upnp_device_t * dev, int port )
+devicePulse( struct upnp_dev * dev, int port )
 {
     const char * body;
     int          len, code;
@@ -820,11 +888,15 @@ makeHttp( int method, const char * host, int port, const char * path )
 }
 
 static tr_http_t *
-devicePulseGetHttp( tr_upnp_device_t * dev )
+devicePulseGetHttp( struct upnp_dev * dev )
 {
     tr_http_t  * ret;
     char         numstr[6];
     const char * type;
+#ifdef VERBOSE_LOG
+    const char * body;
+    int          len;
+#endif
 
     ret = NULL;
     switch( dev->state ) 
@@ -886,11 +958,25 @@ devicePulseGetHttp( tr_upnp_device_t * dev )
             break;
     }
 
+#ifdef VERBOSE_LOG
+    if( NULL != ret )
+    {
+        tr_httpGetHeaders( ret, &body, &len );
+        fprintf( vlog, "send http message, %i bytes (headers):\n", len );
+        fwrite( body, 1, len, vlog );
+        tr_httpGetBody( ret, &body, &len );
+        fprintf( vlog, "\n\nsend http message, %i bytes (body):\n", len );
+        fwrite( body, 1, len, vlog );
+        fputs( "\n\n", vlog );
+        fflush( vlog );
+    }
+#endif
+
     return ret;
 }
 
 static int
-devicePulseHttp( tr_upnp_device_t * dev,
+devicePulseHttp( struct upnp_dev * dev,
                  const char ** body, int * len )
 {
     const char * headers;
@@ -922,6 +1008,12 @@ devicePulseHttp( tr_upnp_device_t * dev,
     switch( tr_httpPulse( dev->http, &headers, &hlen ) )
     {
         case TR_NET_OK:
+#ifdef VERBOSE_LOG
+    fprintf( vlog, "receive http message, %i bytes:\n", hlen );
+    fwrite( headers, 1, hlen, vlog );
+    fputs( "\n\n", vlog );
+    fflush( vlog );
+#endif
             code = tr_httpResponseCode( headers, hlen );
             if( SOAP_METHOD_NOT_ALLOWED == code && !dev->soapretry )
             {
@@ -931,7 +1023,7 @@ devicePulseHttp( tr_upnp_device_t * dev,
             }
             dev->soapretry = 0;
             *body = tr_httpParse( headers, hlen, NULL );
-            *len = ( NULL == body ? 0 : hlen - ( *body - headers ) );
+            *len = ( NULL == *body ? 0 : hlen - ( *body - headers ) );
             return code;
         case TR_NET_ERROR:
             killHttp( &dev->http );
@@ -1028,7 +1120,7 @@ parseRoot( const char * root, const char *buf, int len,
     {
         basedup = strrchr( root, '/' );
         assert( NULL != basedup );
-        basedup = tr_dupstr( root, basedup - root + 1 );
+        basedup = tr_strndup( root, basedup - root + 1 );
     }
     else
     {
@@ -1067,8 +1159,8 @@ addUrlbase( const char * base, char ** path )
 }
 
 static int
-parseScpd( const char * buf, int len, tr_upnp_action_t * getcmd,
-           tr_upnp_action_t * addcmd, tr_upnp_action_t * delcmd )
+parseScpd( const char * buf, int len, struct upnp_act * getcmd,
+           struct upnp_act * addcmd, struct upnp_act * delcmd )
 {
     const char * end, * next, * sub, * name;
 
@@ -1114,7 +1206,7 @@ parseScpd( const char * buf, int len, tr_upnp_action_t * getcmd,
 
 static int
 parseScpdArgs( const char * buf, const char * end,
-               tr_upnp_action_t * action, char dir )
+               struct upnp_act * action, char dir )
 {
     const char * sub, * which;
     char       * name, * var;
@@ -1146,7 +1238,7 @@ parseScpdArgs( const char * buf, const char * end,
 }
 
 static int
-parseMapping( tr_upnp_device_t * dev, const char * buf, int len )
+parseMapping( struct upnp_dev * dev, const char * buf, int len )
 {
     const char * end, * down, * next, * var;
     int          varlen, pret, cret, eret;
@@ -1223,7 +1315,7 @@ joinstrs( const char * first, const char * delim, const char * second )
 
 static tr_http_t *
 soapRequest( int retry, const char * host, int port, const char * path,
-             const char * type, tr_upnp_action_t * action, ... )
+             const char * type, struct upnp_act * action, ... )
 {
     tr_http_t  * http;
     va_list      ap;
@@ -1251,7 +1343,7 @@ soapRequest( int retry, const char * host, int port, const char * path,
 "    xmlns:s=\"" SOAP_ENVELOPE "\""
 "    s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">"
 "  <s:Body>"
-"    <u:%s xmlns:u=\%s\">", action->name, type );
+"    <u:%s xmlns:u=\"%s\">", action->name, type );
 
         va_start( ap, action );
         do
@@ -1287,7 +1379,7 @@ soapRequest( int retry, const char * host, int port, const char * path,
 }
 
 static void
-actionSetup( tr_upnp_action_t * action, const char * name, int prealloc )
+actionSetup( struct upnp_act * action, const char * name, int prealloc )
 {
     action->name = strdup( name );
     assert( NULL == action->args );
@@ -1297,7 +1389,7 @@ actionSetup( tr_upnp_action_t * action, const char * name, int prealloc )
 }
 
 static void
-actionFree( tr_upnp_action_t * act )
+actionFree( struct upnp_act * act )
 {
     free( act->name );
     while( 0 < act->len )
@@ -1310,7 +1402,7 @@ actionFree( tr_upnp_action_t * act )
 }
 
 static int
-actionAdd( tr_upnp_action_t * act, char * name, char * var, char dir )
+actionAdd( struct upnp_act * act, char * name, char * var, char dir )
 {
     int    ii;
     void * newbuf;
@@ -1342,7 +1434,7 @@ actionAdd( tr_upnp_action_t * act, char * name, char * var, char dir )
 }
 
 static const char *
-actionLookup( tr_upnp_action_t * act, const char * key, int len,
+actionLookup( struct upnp_act * act, const char * key, int len,
               char dir, int getname )
 {
     int ii;
@@ -1377,95 +1469,3 @@ actionLookup( tr_upnp_action_t * act, const char * key, int len,
 
     return NULL;
 }
-
-#if 0
-/* this code is used for standalone root parsing for debugging purposes */
-/* cc -g -Wall -D__TRANSMISSION__ -o upnp upnp.c xml.c utils.c */
-int
-main( int argc, char * argv[] )
-{
-    struct stat sb;
-    char      * data, * soap, * scpd;
-    int         fd, ppp;
-    ssize_t     res;
-
-    if( 3 != argc )
-    {
-        printf( "usage: %s root-url root-file\n", argv[0] );
-        return 0;
-    }
-
-    tr_msgInit();
-    tr_setMessageLevel( 9 );
-
-    if( 0 > stat( argv[2], &sb ) )
-    {
-        tr_err( "failed to stat file %s: %s", argv[2], strerror( errno ) );
-        return 1;
-    }
-
-    data = malloc( sb.st_size );
-    if( NULL == data )
-    {
-        tr_err( "failed to malloc %zd bytes", ( size_t )sb.st_size );
-        return 1;
-    }
-
-    fd = open( argv[2], O_RDONLY );
-    if( 0 > fd )
-    {
-        tr_err( "failed to open file %s: %s", argv[2], strerror( errno ) );
-        free( data );
-        return 1;
-    }
-
-    res = read( fd, data, sb.st_size );
-    if( sb.st_size > res )
-    {
-        tr_err( "failed to read file %s: %s", argv[2],
-                ( 0 > res ? strerror( errno ) : "short read count" ) );
-        close( fd );
-        free( data );
-        return 1;
-    }
-
-    close( fd );
-
-    if( parseRoot( argv[1], data, sb.st_size, &soap, &scpd, &ppp ) )
-    {
-        tr_err( "root parsing failed" );
-    }
-    else
-    {
-        tr_err( "soap=%s scpd=%s ppp=%s", soap, scpd, ( ppp ? "yes" : "no" ) );
-        free( soap );
-        free( scpd );
-    }
-    free( data );
-
-    return 0;
-}
-
-int  tr_netMcastOpen( int port, struct in_addr addr ) { assert( 0 ); }
-int  tr_netBind    ( int port, int type ) { assert( 0 ); }
-void tr_netClose   ( int s ) { assert( 0 ); }
-int  tr_netRecvFrom( int s, uint8_t * buf, int size, struct sockaddr_in * sin ) { assert( 0 ); }
-int         tr_httpRequestType( const char * data, int len,
-                                char ** method, char ** uri ) { assert( 0 ); }
-int         tr_httpResponseCode( const char * data, int len ) { assert( 0 ); }
-char *      tr_httpParse( const char * data, int len, tr_http_header_t *headers ) { assert( 0 ); }
-int         tr_httpIsUrl( const char * u, int l ) { assert( 0 ); }
-int         tr_httpParseUrl( const char * u, int l, char ** h, int * p, char ** q ) { assert( 0 ); }
-tr_http_t   * tr_httpClient( int t, const char * h, int p, const char * u, ... ) { assert( 0 ); }
-tr_http_t   * tr_httpClientUrl( int t, const char * u, ... ) { assert( 0 ); }
-void          tr_httpAddHeader( tr_http_t * h, const char * n, const char * v ) { assert( 0 ); }
-void          tr_httpAddBody( tr_http_t * h, const char * b, ... ) { assert( 0 ); }
-tr_tristate_t tr_httpPulse( tr_http_t * h, const char ** b, int * l ) { assert( 0 ); }
-char        * tr_httpWhatsMyAddress( tr_http_t * h ) { assert( 0 ); }
-void          tr_httpClose( tr_http_t * h ) { assert( 0 ); }
-
-void tr_lockInit     ( tr_lock_t * l ) {}
-int  pthread_mutex_lock( pthread_mutex_t * m ) { return 0; }
-int  pthread_mutex_unlock( pthread_mutex_t * m ) { return 0; }
-
-#endif
